package org.rcisoft.app.appNotice.controller;


/*固定导入*/

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import org.rcisoft.business.cmsNotice.entity.CmsNotice;
import org.rcisoft.business.cmsNotice.service.CmsNoticeService;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
 * @author cy
 * @date 2024年3月25日 下午1:42:40
 */
@RestController
@RequestMapping("/app")
public class AppNoticeController extends CyPaginationController<CmsNotice> {


    @Autowired
    private CmsNoticeService cmsNoticeServiceImpl;

    /**
     * 首页-公告详情页
     */
    @PreAuthorize("@cyPerm.hasPerm('app:notice:detail')")
    @CyOpeLogAnno(title = "system-公告管理-查询公告", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "查询单一公告", description = "查询单一公告")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @GetMapping("/open/cmsNotice/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable Integer businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNoticeServiceImpl.findById(businessId));
    }


    /**
     * 首页-查看公告列表
     */
    @PreAuthorize("@cyPerm.hasPerm('app:notice:query')")
    @CyOpeLogAnno(title = "system-公告管理-查询公告", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "分页查询公告集合", description = "分页查询公告集合")
    @GetMapping(value = "/open/cmsNotice/queryCmsNoticeByPagination")
    public CyGridModel listByPagination(CmsNotice cmsNotice) {
        cmsNoticeServiceImpl.findAllByPagination(getPaginationUtility(), cmsNotice);
        return getGridModelResponse();
    }

}
