package org.rcisoft.app.login.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.codehaus.xfire.util.Base64;
import org.rcisoft.app.login.service.WxMiniService;
import org.rcisoft.core.jwt.bean.CyJwtBean;
import org.rcisoft.core.jwt.model.CyJwtUser;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.CyAESUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.util.RedisCons;
import org.rcisoft.sys.rbac.menu.dao.SysMenuRbacRepository;
import org.rcisoft.sys.rbac.menu.dto.SysMenuRbacDTO;
import org.rcisoft.sys.rbac.menu.entity.SysMenuRbac;
import org.rcisoft.sys.rbac.user.dao.SysUserRbacRepository;
import org.rcisoft.sys.rbac.user.entity.SysUserRbac;
import org.rcisoft.sys.wbac.user.entity.SysUser;
import org.rcisoft.tencent.cons.CyWxMiniCons;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.*;
import java.security.spec.InvalidParameterSpecException;
import java.util.*;

/**
 * Created with family.
 * author: cy
 * Date: 2021/5/10
 * Time: 4:10 PM
 * description:
 */
@Service
@Slf4j
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class WxMiniServiceImpl implements WxMiniService {
    // appId
    @Value("${wx.appId:}")
    private String appId;

    // secret
    @Value("${wx.secret:}")
    private String secret;

    @Value("${wx.timeOut:3600}")
    private Integer timeOut;


    @Autowired(required = false)
    private CyRedisService cyRedisServiceImpl;

    @Autowired
    private CyJwtBean cyJwtBean;

    @Autowired
    private CyJwtUtil cyJwtUtil;

    @Autowired
    private SysUserRbacRepository sysUserRbacRepository;

    @Autowired(required = false)
    private SysMenuRbacRepository sysMenuRbacRepository;


    @Override
    public Map<String, Object> validLogin(String code, String token) {
        Map<String, Object> resultMap = new HashMap<>();
        Map<String, String> res = this.getWxOpenId(code);
        String wxOpenid = res.get(CyWxMiniCons.OPEN_ID_STR);
        String sessionKey = res.get(CyWxMiniCons.SESSION_KEY_STR);
        // open is null;
        if (StringUtils.isEmpty(wxOpenid)) {
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_ERROR);
            resultMap.put(CyWxMiniCons.DATA, "未获取到openid");
            return resultMap;
        }
        // redis 存放sessionKey
        cyRedisServiceImpl.set(CyWxMiniCons.REDIS_OPENID_STR + wxOpenid
                , sessionKey, Long.parseLong(timeOut + ""));
        // 1. 已经登录 ----> 返回token 和 基本信息
        if (StringUtils.isNotEmpty(CyUserUtil.getAuthenUsername()) && !cyJwtBean.isTokenExpired(token)) {
            return alreadyLogin(resultMap, token);
        }
        // 2. 未登录
        //2.1.1 根据 open_id 数据库数据查询
        List<SysUserRbac> sysUsers = sysUserRbacRepository.selectByOpenid(wxOpenid);
        if (CollectionUtils.isNotEmpty(sysUsers))
            // 2.1.2 微信登录,数据库已绑定 openId --->
            return wxLogin(resultMap, sysUsers.get(0));
        else {
            // 2.1.3 未登录，数据库未绑定 openId 新用户
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_UNLOGIN);
            resultMap.put(CyWxMiniCons.OPEN_ID, wxOpenid);
            return resultMap;

        }

    }

    @Transactional(propagation = Propagation.REQUIRED)
    @Override
    public Map<String, Object> phoneLogin(String encryptedData, String iv, String openId) {
        Map<String, Object> resultMap = new HashMap<>();
        Object session_key = cyRedisServiceImpl.get(CyWxMiniCons.REDIS_OPENID_STR + openId);
        if (null != session_key && StringUtils.isAnyEmpty(session_key.toString())) {
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_ERROR);
            resultMap.put(CyWxMiniCons.DATA, CyWxMiniCons.OPE_PHONE_ERROR);
            return resultMap;
        }
        //1.解析phone
        JSONObject res = this.getPhoneNumber(session_key.toString(), encryptedData, iv);
        if (res == null || null == res.get(CyWxMiniCons.PHONE_STR) || StringUtils.isAnyEmpty(res.get(CyWxMiniCons.PHONE_STR).toString())) {
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_ERROR);
            resultMap.put(CyWxMiniCons.DATA, CyWxMiniCons.OPE_PHONE_ERROR);
            return resultMap;
        }
        String phone = res.get(CyWxMiniCons.PHONE_STR).toString();
        if (StringUtils.isNotBlank(phone))
            phone = CyAESUtils.encrypt(phone);
        log.error("phone---------------------" + phone);
        List<SysUserRbac> sysUsers = sysUserRbacRepository.selectByPhone(phone);
        //2.1 手机号已存在
        if (CollectionUtils.isNotEmpty(sysUsers))
            return this.wxPhoneLogin(resultMap, sysUsers.get(0), openId, phone);
        //2.2 手机号不存在,客户第一次登录
        return this.wxPhoneLogin(resultMap, null, openId, phone);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    @Override
    public Map<String, Object> saveUserProfile(String nickName, String avatarUrl, String gender, String address, String token) {
        Map<String, Object> resultMap = new HashMap<>();
        String phone = CyUserUtil.getUserInfoStrProp(token, CyUserUtil.USER_PHONE);
        //非法登录
        if (StringUtils.isAnyEmpty(token) &&
                StringUtils.isAnyEmpty(phone)) {
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_ERROR);
            resultMap.put(CyWxMiniCons.DATA, CyWxMiniCons.OPE_PROFILE_NOT_LOGIN);
        }
        SysUserRbac sysUser = new SysUserRbac();
        sysUser.setPhone(phone);
        sysUser.setWxHead(avatarUrl);
        sysUser.setWxSex(gender);
        sysUser.setWxNickName(nickName);
        sysUser.setWxAddress(address);
        sysUserRbacRepository.updateUserWxByPhone(sysUser);
        //重新生成token
        List<SysUserRbac> users = sysUserRbacRepository.selectByPhone(phone);
        if (CollectionUtils.isNotEmpty(users)) {
            SysUserRbac sysUserRbac = users.get(0);
            /*根据用户名查询用户权限*/
            SysMenuRbacDTO sysMenu = new SysMenuRbacDTO();
            sysMenu.setNotDeleted();
            sysMenu.setUsername(sysUserRbac.getUsername());
            List<SysMenuRbac> menus = sysMenuRbacRepository.queryIdByUsername(sysMenu);
            if (CollectionUtils.isEmpty(menus))
                menus = Collections.emptyList();
            sysUserRbac.setMenuList(menus);
            boolean notLocked = true;
            CyJwtUser userDetails = CyJwtUtil.createRbacJwtUser(sysUserRbac, notLocked);
            //创建token
            resultMap.put(CyWxMiniCons.TOKEN, CyJwtUtil.generateToken(userDetails));
        }
        resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_AUTHOR);
        resultMap.put(CyWxMiniCons.DATA, sysUser);
        return resultMap;
    }

    /**
     * 解析phoneNumber
     *
     * @param session_key
     * @param encryptedData
     * @param iv
     * @return
     */
    private JSONObject getPhoneNumber(String session_key, String encryptedData, String iv) {
        byte[] dataByte = Base64.decode(encryptedData);
        byte[] keyByte = Base64.decode(session_key);
        byte[] ivByte = Base64.decode(iv);
        try {

            int base = 16;
            if (keyByte.length % base != 0) {
                int groups = keyByte.length / base + (keyByte.length % base != 0 ? 1 : 0);
                byte[] temp = new byte[groups * base];
                Arrays.fill(temp, (byte) 0);
                System.arraycopy(keyByte, 0, temp, 0, keyByte.length);
                keyByte = temp;
            }
            // 初始化
            Security.addProvider(new BouncyCastleProvider());
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "BC");
            SecretKeySpec spec = new SecretKeySpec(keyByte, "AES");
            AlgorithmParameters parameters = AlgorithmParameters.getInstance("AES");
            parameters.init(new IvParameterSpec(ivByte));
            cipher.init(Cipher.DECRYPT_MODE, spec, parameters);
            byte[] resultByte = cipher.doFinal(dataByte);
            if (null != resultByte && resultByte.length > 0) {
                String result = new String(resultByte, "UTF-8");
                return JSONUtil.parseObj(result);
            }
        } catch (NoSuchAlgorithmException e) {
            log.error(e.getMessage(), e);
        } catch (NoSuchPaddingException e) {
            log.error(e.getMessage(), e);
        } catch (InvalidParameterSpecException e) {
            log.error(e.getMessage(), e);
        } catch (IllegalBlockSizeException e) {
            log.error(e.getMessage(), e);
        } catch (BadPaddingException e) {
            log.error(e.getMessage(), e);
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage(), e);
        } catch (InvalidKeyException e) {
            log.error(e.getMessage(), e);
        } catch (InvalidAlgorithmParameterException e) {
            log.error(e.getMessage(), e);
        }catch (NoSuchProviderException e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * 登录获取sessionKey
     *
     * @param code
     * @return
     */
    private Map<String, String> getWxOpenId(String code) {
        Map<String, String> res = new HashMap<>();
        String loginUrl = String.format(CyWxMiniCons.LOGIN_URL, appId, secret, code);
        CloseableHttpClient client = null;
        CloseableHttpResponse response = null;
        try {
            // 创建http GET请求
            HttpGet httpGet = new HttpGet(loginUrl);
            client = HttpClients.createDefault();
            // 执行请求
            response = client.execute(httpGet);
            HttpEntity entity = response.getEntity();//得到返回数据
            String result = EntityUtils.toString(entity);
            log.error("微信返回---------------------" + result);
            JSONObject json_test = JSONUtil.parseObj(result);
            String wxOpenid = json_test.getStr(CyWxMiniCons.OPEN_ID_STR);
            String sessionKey = json_test.getStr(CyWxMiniCons.SESSION_KEY_STR);
            log.error("openId---------------------" + wxOpenid);
            log.error("sessionKey---------------------" + sessionKey);
            res.put(CyWxMiniCons.OPEN_ID_STR, wxOpenid);
            res.put(CyWxMiniCons.SESSION_KEY_STR, sessionKey);
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    log.error(e.getMessage());
                }
            }
            if (client != null) {
                try {
                    client.close();
                } catch (IOException e) {
                    log.error(e.getMessage());
                }
            }
        }
        return res;
    }

    /**
     * 手机登录
     *
     * @param resultMap
     * @return
     */
    private Map<String, Object> wxPhoneLogin(Map<String, Object> resultMap, SysUserRbac sysUserRbac, String openId, String phone) {
        //2.1.1 第一次登录
        if (null == sysUserRbac) {
            // 手机号为帐号，验证重, username phone
            List<SysUserRbac> list = sysUserRbacRepository.queryUserByUserName(phone);
            if (CollectionUtils.isNotEmpty(list)) {
                resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_ERROR);
                resultMap.put(CyWxMiniCons.DATA, CyWxMiniCons.OPE_PHONE_EXIST);
                return resultMap;
            }
            sysUserRbac = new SysUserRbac();
            sysUserRbac.setWxOpenid(openId);
            sysUserRbac.setPhone(phone);
            sysUserRbac.setUsername(CyAESUtils.decrypt(phone));
            sysUserRbac.setCreateDate(new Date());
            sysUserRbac.setUpdateDate(new Date());
            sysUserRbac.setNotDeleted();
            sysUserRbac.setStart();
            sysUserRbacRepository.insert(sysUserRbac);
            //将会员信息存入redis
            SysUser user = new SysUser();
            BeanUtil.copyProperties(sysUserRbac, user);
            cyRedisServiceImpl.set(RedisCons.MEMBER_INFO +":"+ sysUserRbac.getBusinessId(),user);
        }
        //2.1.2 openId 是否绑定   情况：用户旧的微信注销了，更换了微信，但是绑定的手机号都是相同的
        else if (StringUtils.isNotEmpty(sysUserRbac.getWxOpenid()) && !sysUserRbac.getWxOpenid().equals(openId)) {
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_ERROR);
            resultMap.put(CyWxMiniCons.DATA, CyWxMiniCons.OPE_PHONE_EXIST);
            return resultMap;
        }
        //2.1.3 openId 未绑定
        else if (StringUtils.isAnyBlank(sysUserRbac.getWxOpenid())) {
            SysUserRbac sysUser = new SysUserRbac();
            sysUser.setPhone(phone);
            sysUser.setWxOpenid(openId);
            sysUserRbacRepository.updateUserWxByPhone(sysUser);
        }
        return this.wxLogin(resultMap, sysUserRbac);
    }

    /**
     * wx登录
     *
     * @param resultMap
     * @return
     */
    private Map<String, Object> wxLogin(Map<String, Object> resultMap, SysUserRbac sysUserRbac) {
        if (null != sysUserRbac.getWxNickName())
            // 1.1 已授权
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_AUTHOR);
        else {
            // 1.2 已登录，未授权
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_LOGIN);
            //resultMap.put(WxMiniCons.OPEN_ID, sysUserRbac.getWxOpenid());
        }
        /*根据用户名查询用户权限*/
        SysMenuRbacDTO sysMenu = new SysMenuRbacDTO();
        sysMenu.setNotDeleted();
        sysMenu.setUsername(sysUserRbac.getUsername());
        List<SysMenuRbac> menus = sysMenuRbacRepository.queryIdByUsername(sysMenu);
        if (CollectionUtils.isEmpty(menus))
            menus = Collections.emptyList();
        sysUserRbac.setMenuList(menus);
        boolean notLocked = true;
        CyJwtUser userDetails = CyJwtUtil.createRbacJwtUser(sysUserRbac, notLocked);
        //创建token
        resultMap.put(CyWxMiniCons.TOKEN, CyJwtUtil.generateToken(userDetails));
        sysUserRbac.setPassword(null);
        resultMap.put(CyWxMiniCons.DATA, sysUserRbac);
        return resultMap;
    }

    /**
     * 已经登录
     *
     * @param resultMap
     * @return
     */
    private Map<String, Object> alreadyLogin(Map<String, Object> resultMap, String token) {
        cyJwtUtil.refreshToken(token);
        if (null != CyUserUtil.getUserInfoStrProp(token, CyUserUtil.USER_WX_NICKNAME))
            // 1.1 已授权
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_AUTHOR);
        else {
            // 1.2 已登录，未授权
            resultMap.put(CyWxMiniCons.CODE, CyWxMiniCons.CODE_LOGIN);
        }
        resultMap.put(CyWxMiniCons.TOKEN, token);
        List<SysUserRbac> users = sysUserRbacRepository.queryUserByUserName(CyUserUtil.getAuthenUsername());
        SysUserRbac sysUserRbac = new SysUserRbac();
        if (CollectionUtils.isNotEmpty(users)) {
            sysUserRbac = users.get(0);
            sysUserRbac.setPassword(null);
        }
        //resultMap.put(WxMiniCons.DATA, CyUserUtil.getUserInfoStrProp(token, CyUserUtil.USER_PHONE));
        resultMap.put(CyWxMiniCons.DATA, sysUserRbac);
        return resultMap;
    }
}
