package org.rcisoft.app.pay.controller;

import lombok.extern.slf4j.Slf4j;
import org.jdom.JDOMException;
import org.rcisoft.app.pay.service.WxPayService;
import org.rcisoft.business.cmsOrder.dto.OrderActivityDto;
import org.rcisoft.business.cmsOrder.entity.CmsOrder;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;

/**
 * @author cy
 * Created by cy on 21/5/10.
 * @descript
 */
@RestController
@RequestMapping("/app/wxPay")
@Slf4j
public class WxPayController {

    @Autowired
    private WxPayService wxPayService;

    @Autowired
    private CyJwtUtil cyJwtUtil;


    /**
     * 创建微信预支付订单
     *
     * @return
     */
    @PostMapping("/pay")
    public Map<String, Object> pay(HttpServletRequest request,@RequestBody OrderActivityDto orderActivityDto) throws IOException, JDOMException {
        return wxPayService.wxPay(orderActivityDto.getOrderId(), orderActivityDto.getActivityId(), request);
    }

    /**
     * 小程序调用wx.requestPayment(OBJECT)发起微信支付成功后更新订单状态
     *
     * @return
     */
    @PostMapping("/updateOrderStatus")
    public CyResult updateOrderStatus(@RequestBody OrderActivityDto orderActivityDto) {
        CyPersistModel data = wxPayService.updateOrderStatus(orderActivityDto.getOrderId(), orderActivityDto.getActivityId());
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                data);
    }

    //微信异步回调通知支付
    @RequestMapping("/notifyCallBack")
    @ResponseBody
    public void notifyCallBack(HttpServletRequest request, HttpServletResponse response)  throws IOException, JDOMException {
        wxPayService.wxNotify(request,response);
    }

    //微信异步回调通知退款
    @RequestMapping("/notifyRefundCallBack")
    @ResponseBody
    public void notifyRefundCallBack(HttpServletRequest request, HttpServletResponse response)  throws Exception {
        wxPayService.notifyRefundCallBack(request,response);
    }

    @PostMapping("/test")
    public CyResult test(@RequestBody CmsOrder orderInfo) {
        String data = wxPayService.refundOrder(orderInfo);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                data);
    }
}
