package org.rcisoft.business.cmsActivity.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.cmsActivity.dao.CmsActivityRepository;
import org.rcisoft.business.cmsActivity.entity.CmsActivity;
import org.rcisoft.business.cmsActivity.service.CmsActivityService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.util.RedisCons;
import org.rcisoft.core.util.TimeUtil;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @author cy
 * @date 2024年3月25日 下午1:42:40
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CmsActivityServiceImpl extends ServiceImpl<CmsActivityRepository, CmsActivity> implements CmsActivityService {

    @Autowired(required = false)
    private CyRedisService cyRedisServiceImpl;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 保存 活动
     *
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(CmsActivity cmsActivity) {
        //增加操作
        cmsActivity.setDelFlag("0");
        //当图片字段没有时增加默认1
/*        if (ObjectUtil.isNull(cmsActivity.getPictureId())) {
            cmsActivity.setPictureId(global.getDEFAULT_NEWS_LOCATION());
        }*/
        CmsActivity check = baseMapper.checkWeight(cmsActivity.getWeight());
        if (ObjectUtil.isNotNull(check)) {
            throw new CyServiceException(500, "权重已存在");
        } else {
            int line = baseMapper.insert(cmsActivity);
            log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                    cmsActivity.getBusinessId() + "的活动信息");
            return new CyPersistModel(line);
        }
    }

    /**
     * 删除 活动
     *
     * @param cmsActivity
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(CmsActivity cmsActivity) {
        int line = baseMapper.deleteCmsActivity(cmsActivity.getBusinessId());
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                cmsActivity.getBusinessId() + "的活动信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 活动
     *
     * @param cmsActivity
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(CmsActivity cmsActivity) {
        //修改文本中的#image
        String details = cmsActivity.getDetails().replace("#image", "data:image");
        cmsActivity.setDetails(details);
        CmsActivity check = baseMapper.checkWeight(cmsActivity.getWeight());
        if (ObjectUtil.isNotNull(check) && !check.getBusinessId().equals(cmsActivity.getBusinessId())) {
            throw new CyServiceException(500, "权重已存在");
        } else {
            int line = baseMapper.updateById(cmsActivity);
            log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsActivity.getBusinessId() + "的活动信息");
            return new CyPersistModel(line);
        }
    }

    /**
     * 根据id查询 活动
     *
     * @param id
     * @return
     */
    @Override
    public CmsActivity findById(Integer id) {
        CmsActivity cmsActivity = baseMapper.selectById(id);
        return cmsActivity;
    }

    /**
     * 分页查询 活动
     *
     * @param cmsActivity
     * @return
     */
    @Override
    public IPage<CmsActivity> findAllByPagination(CyPageInfo<CmsActivity> paginationUtility,
                                                  CmsActivity cmsActivity) {
        IPage<CmsActivity> cmsActivityIPage = baseMapper.queryCmsActivityPaged(paginationUtility, cmsActivity);
        List<CmsActivity> newsInformation = cmsActivityIPage.getRecords();
        cmsActivityIPage.setRecords(newsInformation);
        return cmsActivityIPage;

    }

    /**
     * 查询list 活动
     *
     * @param cmsActivity
     * @return
     */
    @Override
    public List<CmsActivity> findAll(CmsActivity cmsActivity) {
        return baseMapper.queryCmsActivity(cmsActivity);

    }

    /**
     * 导出活动
     *
     * @return
     */
    @Override
    public List<CmsActivity> export(CmsActivity cmsActivity) {
        List<CmsActivity> cmsActivityList = baseMapper.queryCmsActivity(cmsActivity);
        return cmsActivityList;
    }

    @Override
    public CyPersistModel updateStatus(CmsActivity cmsActivity) {
        int line = baseMapper.updateStatus(cmsActivity);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsActivity.getBusinessId() + "的活动信息状态");
        return new CyPersistModel(line);
    }

    /**
     * 修改活动
     *
     * @return 结果
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel pushActivityInfo(Long businessId,String status)
    {
        CmsActivity info = baseMapper.selectById(businessId);
        int line = 0;
        //如果是发布操作，需要校验
        if ("1".equals(status)){
            if ("1".equals(info.getPublishStatus()))
                throw new CyServiceException("活动不是未发布状态，不能发布");
            info.setUpdateDate(new Date());
            info.setUpdateBy(CyUserUtil.getAuthenBusinessId());
            info.setPublishDate(new Date());
            info.setPublishStatus(status);
            line = baseMapper.updateById(info);
            Date endTime = TimeUtil.getTime(info.getEndTime(), Calendar.DATE,1);
            //将活动信息插入redis
            long time = TimeUtil.getTimeSecond(new Date(),endTime);
            cyRedisServiceImpl.set(RedisCons.ACTIVITY_DETAIL +":"+ businessId,info,time);
            cyRedisServiceImpl.set(RedisCons.ACTIVITY_COUNT +":"+ businessId,info.getMaxApplicationCount(),time);
        } else {
            //如果取消发布，加redis锁
            boolean isGetLock = false;
            RLock lock = redissonClient.getLock(RedisCons.ACTIVITY_INFO + businessId);
            try {
                isGetLock = lock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                        Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                if (isGetLock) {
                    //校验活动是否已经报名，状态是否为发布
                    extracted(businessId, info);
                    info.setUpdateDate(new Date());
                    info.setUpdateBy(CyUserUtil.getAuthenBusinessId());
                    info.setPublishStatus(status);
                    line = baseMapper.updateById(info);
                    cyRedisServiceImpl.del(RedisCons.ACTIVITY_DETAIL +":"+ businessId);
                    cyRedisServiceImpl.del(RedisCons.ACTIVITY_COUNT +":"+ businessId);
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                isGetLock = false;
            } finally {
                lock.unlock();
                log.error("释放成功");
            }
        }
        return new CyPersistModel(line);
    }

    private void extracted(Long businessId, CmsActivity info) {
        Set<String> map = redisTemplate.keys(RedisCons.ORDER_ACTIVITY + ":" + businessId + "*");
        if (CollectionUtils.isNotEmpty(map)) //报名人数和实际剩余报名人数不等
            throw new CyServiceException("活动已经报名，不允许取消发布");
        if ("0".equals(info.getPublishStatus()))
            throw new CyServiceException("活动未发布，不能取消");
    }
}
