package org.rcisoft.business.cmsApplication.controller;
/*固定导入*/

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import org.rcisoft.business.cmsApplication.entity.CmsApplication;
import org.rcisoft.business.cmsApplication.service.CmsApplicationService;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @author cy
 * @date 2024年3月25日 下午1:42:40
 */
@RestController
@RequestMapping("/cmsApplication")
public class CmsApplicationController extends CyPaginationController<CmsApplication> {


    @Autowired
    private CmsApplicationService cmsApplicationServiceImpl;

    @PreAuthorize("@cyPerm.hasPerm('cms:application:increase')")
    @CyOpeLogAnno(title = "system-报名表管理-新增报名表", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary = "添加报名表", description = "添加报名表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid @RequestBody CmsApplication cmsApplication, BindingResult bindingResult) {
        CyPersistModel data = cmsApplicationServiceImpl.persist(cmsApplication);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsApplication);
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:activity:removing')")
    @CyOpeLogAnno(title = "system-报名表管理-删除报名表", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary = "删除报名表", description = "删除报名表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable Integer businessId, CmsApplication cmsApplication) {
        cmsApplication.setBusinessId(businessId);
        CyPersistModel data = cmsApplicationServiceImpl.remove(cmsApplication);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:activity:modifications')")
    @CyOpeLogAnno(title = "system-报名表管理-修改报名表", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary = "修改报名表", description = "修改报名表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false)})
    @PostMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable Integer businessId, @Valid CmsApplication cmsApplication, BindingResult bindingResult) {
        cmsApplication.setBusinessId(businessId);
        CyPersistModel data = cmsApplicationServiceImpl.merge(cmsApplication);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsApplication);
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:application:singleSearch')")
    @CyOpeLogAnno(title = "system-报名管理-查询报名", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "查询单一报名", description = "查询单一报名")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable Integer businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsApplicationServiceImpl.findById(businessId));
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:application:list')")
    @CyOpeLogAnno(title = "system-报名管理-查询报名管理", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "查询报名集合", description = "查询报名集合")
    @GetMapping(value = "/queryCmsApplication")
    public CyResult queryCmsApplication(CmsApplication cmsApplication) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsApplicationServiceImpl.findAll(cmsApplication));
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:application:pageSearch')")
    @CyOpeLogAnno(title = "system-报名管理-查询报名", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "分页查询报名集合", description = "分页查询报名集合")
    @GetMapping(value = "/queryCmsApplicationByPagination")
    public CyGridModel listByPagination(@RequestParam(required = false) Integer activityId, CmsApplication cmsApplication) {
        // 如果传入了 activityId，设置到 cmsApplication 对象中
        if (activityId != null) {
            cmsApplication.setActivityId(activityId);
        }

        // 调用分页查询的方法
        cmsApplicationServiceImpl.findAllByPagination(getPaginationUtility(), cmsApplication);

        // 返回分页结果
        return getGridModelResponse();
    }
}
