package org.rcisoft.business.cmsApplication.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.cmsApplication.entity.CmsApplication;
import org.rcisoft.business.cmsApplication.dao.CmsApplicationRepository;
import org.rcisoft.business.cmsApplication.service.CmsApplicationService;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.*;
import org.rcisoft.sys.wbac.user.dao.SysUserRepository;
import org.rcisoft.sys.wbac.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author cy
 * @date 2024年3月25日 下午1:42:40
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CmsApplicationServiceImpl extends ServiceImpl<CmsApplicationRepository, CmsApplication> implements CmsApplicationService {

    @Autowired
    private SysUserRepository sysUserRepository;

    /**
     * 保存 活动
     *
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(CmsApplication cmsApplication) {
        //获取当前登录人的id
        String userId = CyUserUtil.getAuthenBusinessId();
        cmsApplication.setUserId(Integer.valueOf(userId));
        //根据userId查询用户的name和phone
        SysUser sysUser = sysUserRepository.selectById(userId);
        cmsApplication.setName(sysUser.getName());
        cmsApplication.setPhone(sysUser.getPhone());
        int line = baseMapper.insert(cmsApplication);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
                cmsApplication.getBusinessId()+"的报名表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 活动
     *
     * @param cmsApplication
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(CmsApplication cmsApplication) {
        int line = baseMapper.deleteCmsApplication(cmsApplication.getBusinessId());
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                cmsApplication.getBusinessId() + "的活动信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 活动
     *
     * @param cmsApplication
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(CmsApplication cmsApplication) {
        int line = baseMapper.updateById(cmsApplication);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsApplication.getBusinessId() + "的活动信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 报名表
     *
     * @param id
     * @return
     */
    @Override
    public CmsApplication findById(Integer id) {
        CmsApplication cmsApplication = baseMapper.selectById(id);
        return cmsApplication;
    }

    /**
     * 分页查询 报名表
     *
     * @param cmsApplication
     * @return
     */
    @Override
    public IPage<CmsApplication> findAllByPagination(CyPageInfo<CmsApplication> paginationUtility, CmsApplication cmsApplication) {
        IPage<CmsApplication> cmsApplicationIPage = baseMapper.queryCmsApplicationPaged(paginationUtility, cmsApplication);
        return cmsApplicationIPage;
    }

    /**
     * 查询list 报名表
     *
     * @param cmsApplication
     * @return
     */
    @Override
    public List<CmsApplication> findAll(CmsApplication cmsApplication) {
        return baseMapper.queryCmsApplication(cmsApplication);

    }

}
