package org.rcisoft.business.cmsNotice.controller;


/*固定导入*/

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import org.rcisoft.business.cmsNotice.entity.CmsNotice;
import org.rcisoft.business.cmsNotice.service.CmsNoticeService;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import jakarta.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @author cy
 * @date 2024年3月25日 下午1:42:40
 */
@RestController
@RequestMapping("/cmsNotice")
public class CmsNoticeController extends CyPaginationController<CmsNotice> {


    @Autowired
    private CmsNoticeService cmsNoticeServiceImpl;

    @PreAuthorize("@cyPerm.hasPerm('cms:notice:increase')")
    @CyOpeLogAnno(title = "system-公告管理-新增公告", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary = "添加公告", description = "添加公告")
    @PostMapping(value = "/add")
    public CyResult add(@Valid CmsNotice cmsNotice, BindingResult bindingResult) {
        CyPersistModel data = cmsNoticeServiceImpl.persist(cmsNotice);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNotice);
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:notice:removing')")
    @CyOpeLogAnno(title = "system-公告管理-删除公告", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary = "删除公告", description = "删除公告")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable Integer businessId, CmsNotice cmsNotice) {
        cmsNotice.setBusinessId(businessId);
        CyPersistModel data = cmsNoticeServiceImpl.remove(cmsNotice);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:notice:modifications')")
    @CyOpeLogAnno(title = "system-公告管理-修改公告", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary = "修改公告",  description = "修改公告")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false)})
    @PostMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable Integer businessId, @Valid CmsNotice cmsNotice, BindingResult bindingResult) {
        cmsNotice.setBusinessId(businessId);
        CyPersistModel data = cmsNoticeServiceImpl.merge(cmsNotice);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNotice);
    }

//    @PreAuthorize("@cyPerm.hasPerm('cms:notice:singleSearch')")
    @CyOpeLogAnno(title = "system-公告管理-查询公告", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "查询单一公告", description = "查询单一公告")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable Integer businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNoticeServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:contentNewsInformation:list')")
    @CyOpeLogAnno(title = "system-公告管理-查询公告", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "查询公告集合", description = "查询公告集合")

    @GetMapping(value = "/queryCmsNotice")
    public CyResult querySysContentNewsInformations(CmsNotice cmsNotice) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNoticeServiceImpl.findAll(cmsNotice));
    }

//    @PreAuthorize("@cyPerm.hasPerm('cms:notice:pageSearch')")
    @CyOpeLogAnno(title = "system-公告管理-查询公告", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "分页查询公告集合", description = "分页查询公告集合")
    @GetMapping(value = "/queryCmsNoticeByPagination")
    public CyGridModel listByPagination(CmsNotice cmsNotice) {
        cmsNoticeServiceImpl.findAllByPagination(getPaginationUtility(), cmsNotice);
        return getGridModelResponse();
    }

    //修改状态
    @PreAuthorize("@cyPerm.hasPerm('cms:notice:modificationsStatus')")
    @CyOpeLogAnno(title = "system-公告管理-修改公告状态", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary = "修改公告状态", description = "修改公告状态")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @PostMapping("/updateStatus/{businessId:\\w+}")
    public CyResult updateStatus(CmsNotice cmsNotice) {
        CyPersistModel data = cmsNoticeServiceImpl.updateStatus(cmsNotice);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNotice);
    }
    @PreAuthorize("@cyPerm.hasPerm('cms:notice:derive')")
    @CyOpeLogAnno(title = "system-公告管理-查询公告", businessType = CyLogTypeEnum.EXPORT)
    @Operation(summary = "导出公告信息", description = "导出公告信息")
    @GetMapping(value = "/export")
    public void outSysContentNewsInformation(HttpServletResponse response, CmsNotice cmsNotice, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "公告信息.xls";
                break;
            case "1":
                excelName = "公告信息.xlsx";
                break;
            case "2":
                excelName = "公告信息.csv";
                break;
        }
        List<CmsNotice> cmsNoticeList = cmsNoticeServiceImpl.export(cmsNotice);
        CyEpExcelUtil.exportExcel(cmsNoticeList, "公告信息", "公告信息", CmsNotice.class, excelName, response);
    }
}
