package org.rcisoft.business.cmsNotice.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.cmsNotice.dao.CmsNoticeRepository;
import org.rcisoft.business.cmsNotice.entity.CmsNotice;
import org.rcisoft.business.cmsNotice.service.CmsNoticeService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author cy
 * @date 2024年3月25日 下午1:42:40
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CmsNoticeServiceImpl extends ServiceImpl<CmsNoticeRepository, CmsNotice> implements CmsNoticeService {




    /**
     * 保存 公告
     *
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(CmsNotice cmsNotice) {
        //增加操作
        cmsNotice.setDelFlag("0");
        //当图片字段没有时增加默认1
/*        if (ObjectUtil.isNull(cmsNotice.getPictureId())) {
            cmsNotice.setPictureId(global.getDEFAULT_NEWS_LOCATION());
        }*/
        CmsNotice check = baseMapper.checkWeight(cmsNotice.getWeight());
        if (ObjectUtil.isNotNull(check)) {
            throw new CyServiceException(500, "权重已存在");
        } else {
            int line = baseMapper.insert(cmsNotice);
            log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                    cmsNotice.getBusinessId() + "的公告信息");
            return new CyPersistModel(line);
        }
    }

    /**
     * 删除 公告
     *
     * @param cmsNotice
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(CmsNotice cmsNotice) {
        int line = baseMapper.deleteCmsNotice(cmsNotice.getBusinessId());
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                cmsNotice.getBusinessId() + "的公告信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 公告
     *
     * @param cmsNotice
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(CmsNotice cmsNotice) {
        //修改文本中的#image
        String details = cmsNotice.getDetails().replace("#image", "data:image");
        cmsNotice.setDetails(details);
        CmsNotice check = baseMapper.checkWeight(cmsNotice.getWeight());
        if (ObjectUtil.isNotNull(check) && !check.getBusinessId().equals(cmsNotice.getBusinessId())) {
            throw new CyServiceException(500, "权重已存在");
        } else {
            int line = baseMapper.updateById(cmsNotice);
            log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsNotice.getBusinessId() + "的公告信息");
            return new CyPersistModel(line);
        }
    }

    /**
     * 根据id查询 公告
     *
     * @param id
     * @return
     */
    @Override
    public CmsNotice findById(Integer id) {
        CmsNotice cmsNotice = baseMapper.selectById(id);
        return cmsNotice;
    }

    /**
     * 分页查询 公告
     *
     * @param cmsNotice
     * @return
     */
    @Override
    public IPage<CmsNotice> findAllByPagination(CyPageInfo<CmsNotice> paginationUtility,
                                                CmsNotice cmsNotice) {
        IPage<CmsNotice> cmsNoticeIPage = baseMapper.queryCmsNoticePaged(paginationUtility, cmsNotice);
        List<CmsNotice> newsInformation = cmsNoticeIPage.getRecords();
        cmsNoticeIPage.setRecords(newsInformation);
        return cmsNoticeIPage;

    }

    /**
     * 查询list 公告
     *
     * @param cmsNotice
     * @return
     */
    @Override
    public List<CmsNotice> findAll(CmsNotice cmsNotice) {
        return baseMapper.queryCmsNotice(cmsNotice);

    }

    /**
     * 导出公告
     *
     * @return
     */
    @Override
    public List<CmsNotice> export(CmsNotice cmsNotice) {
        List<CmsNotice> cmsNoticeList = baseMapper.queryCmsNotice(cmsNotice);
        return cmsNoticeList;
    }

    @Override
    public CyPersistModel updateStatus(CmsNotice cmsNotice) {
        int line = baseMapper.updateStatus(cmsNotice);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsNotice.getBusinessId() + "的公告信息状态");
        return new CyPersistModel(line);
    }

}
