package org.rcisoft.business.cmsOrder.controller;


/*固定导入*/
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.Operation;
import org.jdom.JDOMException;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import jakarta.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.business.cmsOrder.entity.CmsOrder;
import org.rcisoft.business.cmsOrder.service.CmsOrderService;

import java.io.IOException;
import java.util.List;

/**
* Created by cy on 2025年1月6日 下午1:44:44.
*/
@RestController
@RequestMapping("/cmsOrder")
public class CmsOrderController extends CyPaginationController<CmsOrder> {


    @Autowired
    private CmsOrderService cmsOrderServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:order:add')")
//    @CyOpeLogAnno(title = "system-订单信息表管理-新增订单信息表", businessType = CyLogTypeEnum.INSERT)
//    @Operation(summary="添加订单信息表", description="添加订单信息表")
//    @PostMapping(value = "/add")
//    public CyResult add(@Valid CmsOrder cmsOrder, BindingResult bindingResult) {
//        CyPersistModel data = cmsOrderServiceImpl.persist(cmsOrder);
//        return CyResultGenUtil.builder(data,
//            CyMessCons.MESSAGE_ALERT_SUCCESS,
//            CyMessCons.MESSAGE_ALERT_ERROR,
//        cmsOrder);
//    }
    @PreAuthorize("@cyPerm.hasPerm('sys:order:delete')")
    @CyOpeLogAnno(title = "system-订单信息表管理-删除订单信息表", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary="逻辑删除订单信息表", description="逻辑删除订单信息表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable Long businessId,CmsOrder cmsOrder) {
        cmsOrder.setBusinessId(businessId);
        CyPersistModel data = cmsOrderServiceImpl.removeLogical(cmsOrder);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }
    @PreAuthorize("@cyPerm.hasPerm('sys:order:delete')")
    @CyOpeLogAnno(title = "system-订单信息表管理-删除订单信息表", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary="删除订单信息表", description="删除订单信息表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable Long businessId,CmsOrder cmsOrder) {
        cmsOrder.setBusinessId(businessId);
        CyPersistModel data = cmsOrderServiceImpl.remove(cmsOrder);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:order:update')")
    @CyOpeLogAnno(title = "system-订单信息表管理-修改订单信息表", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary="修改订单信息表", description="修改订单信息表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false)})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable Long businessId, @Valid CmsOrder cmsOrder, BindingResult bindingResult) {
        cmsOrder.setBusinessId(businessId);
        CyPersistModel data = cmsOrderServiceImpl.merge(cmsOrder);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        cmsOrder);
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:order:query')")
    @CyOpeLogAnno(title = "system-订单信息表管理-查询订单信息表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询单一订单信息表", description="查询单一订单信息表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable String businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        cmsOrderServiceImpl.findById(businessId));
    }

//    @PreAuthorize("@cyPerm.hasPerm('sys:order:list')")
    @CyOpeLogAnno(title = "system-订单信息表管理-查询订单信息表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询订单信息表集合", description="查询订单信息表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(CmsOrder cmsOrder) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            cmsOrderServiceImpl.findAll(cmsOrder));
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:order:list')")
    @CyOpeLogAnno(title = "system-订单信息表管理-查询订单信息表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="分页查询订单信息表集合", description="分页查询订单信息表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(CmsOrder cmsOrder) {
        cmsOrderServiceImpl.findAllByPagination(getPaginationUtility(), cmsOrder);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-订单信息表管理-查询订单信息表", businessType = CyLogTypeEnum.EXPORT)
    @Operation(summary = "导出订单信息表信息", description = "导出订单信息表信息")
    @GetMapping(value = "/export")
    public void outCmsOrder(HttpServletResponse response,CmsOrder cmsOrder,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="订单信息表信息.xls";break;
            case "1": excelName="订单信息表信息.xlsx";break;
            case "2": excelName="订单信息表信息.csv";break;
        }
        List<CmsOrder> cmsOrderList = cmsOrderServiceImpl.export(cmsOrder);
        CyEpExcelUtil.exportExcel(cmsOrderList, "订单信息表信息", "订单信息表信息", CmsOrder.class, excelName, response);
    }

    @CyOpeLogAnno(title = "system-订单管理-退款确认", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary = "退款确认", description = "退款确认")
    @PostMapping(value = "/order/refundConfirm")
    public CyResult refundConfirm(@RequestBody CmsOrder orderInfo) throws IOException, JDOMException {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsOrderServiceImpl.refundConfirm(orderInfo));
    }

    @CyOpeLogAnno(title = "system-订单管理-支付确认", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary = "支付确认", description = "支付确认")
    @PostMapping(value = "/order/payConfirm")
    public CyResult payConfirm(@RequestBody CmsOrder orderInfo)throws IOException, JDOMException {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsOrderServiceImpl.payConfirm(orderInfo));
    }



    @CyOpeLogAnno(title = "system-订单管理-活动退款补偿", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary = "活动退款补偿", description = "活动退款补偿")
    @GetMapping(value = "/refundReimbursement")
    public CyResult refundReimbursement(@RequestParam List<Long> ids) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsOrderServiceImpl.refundReimbursement(ids));
    }

    /**
     * 查询异常订单[根据搜索条件，时间范围跨度3个月，时间最早支持30天前)
     */
    @Operation(summary="查询异常订单列表", description="查询异常订单列表")
    @GetMapping("/selectErrorOrderByPagination")
    public CyGridModel selectErrorOrderByPagination(CmsOrder orderInfo)
    {
        cmsOrderServiceImpl.selectErrorOrderByPagination(getPaginationUtility(), orderInfo);
        return getGridModelResponse();
    }


    @CyOpeLogAnno(title = "system-订单管理-异常订单手动补偿", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary = "异常订单手动补偿", description = "异常订单手动补偿")
    @PostMapping(value = "/manualCompensateOrder")
    public CyResult manualCompensateOrder(@RequestBody CmsOrder dto) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsOrderServiceImpl.manualCompensateOrder(dto));
    }
}
