package org.rcisoft.business.cmsOrder.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.jdom.JDOMException;
import org.rcisoft.business.cmsOrder.entity.CmsOrder;
import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPageInfo;

import java.io.IOException;
import java.util.List;

/**
 * Created by cy on 2025年1月6日 下午1:44:44.
 */
public interface CmsOrderService {

    /**
    * 保存 订单信息表
    * @param cmsOrder
    * @return
    */
    CyPersistModel persist(CmsOrder cmsOrder);

    /**
    * 删除 订单信息表
    * @param cmsOrder
    * @return
    */
    CyPersistModel remove(CmsOrder cmsOrder);

    /**
    * 逻辑删除 订单信息表
    * @param cmsOrder
    * @return
    */
    CyPersistModel removeLogical(CmsOrder cmsOrder);

    /**
    * 修改 订单信息表
    * @param cmsOrder
    * @return
    */
    CyPersistModel merge(CmsOrder cmsOrder);

    /**
    * 根据id查询 订单信息表
    * @param id
    * @return
    */
    CmsOrder findById(String id);

    /**
    * 分页查询 订单信息表
    * @param cmsOrder
    * @return
    */
    IPage<CmsOrder> findAllByPagination(CyPageInfo<CmsOrder> paginationUtility,
              CmsOrder cmsOrder);


    /**
    * 查询list 订单信息表
    * @param cmsOrder
    * @return
    */
    List<CmsOrder> findAll(CmsOrder cmsOrder);


    /**
    * 导出订单信息表
    * @return
    */
    List<CmsOrder> export(CmsOrder cmsOrder);

    /**
     * 活动取消
     * @param orderInfo
     * @return
     */
    int cancel(CmsOrder orderInfo);

    /**
     * 查询会员我的订单列表
     *
     * @param orderInfo 订单
     * @return 订单集合
     */
    public IPage<CmsOrder> selectOrderUserByPagination(CyPageInfo<CmsOrder> paginationUtility, CmsOrder orderInfo);

    /**
     * 退款确认
     * @param orderInfo
     * @return
     */
    int refundConfirm(CmsOrder orderInfo) throws IOException, JDOMException;


    /**
     * 支付确认
     * @param orderInfo
     * @return
     */
    int payConfirm(CmsOrder orderInfo)throws IOException, JDOMException;

    /**
     * 查询异常订单列表
     *
     * @param orderInfo 订单
     * @return 订单集合
     */
    public IPage<CmsOrder> selectErrorOrderByPagination(CyPageInfo<CmsOrder> paginationUtility, CmsOrder orderInfo);


    /**
     * 活动退款补偿
     * @param ids
     * @return
     */
    int refundReimbursement(List<Long> ids);


    /**
     * 异常订单手动补偿
     * @param dto
     * @return
     */
    int manualCompensateOrder(CmsOrder dto);
}
