package org.rcisoft.business.cmsOrder.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.jdom.JDOMException;
import org.rcisoft.app.pay.service.WxPayService;
import org.rcisoft.business.cmsActivity.dao.CmsActivityRepository;
import org.rcisoft.business.cmsActivity.entity.CmsActivity;
import org.rcisoft.business.cmsActivity.service.impl.StockService;
import org.rcisoft.business.cmsOrder.dto.OrderActivityDto;
import org.rcisoft.core.async.AsyncFactory;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.rabbitmq.bean.RabbitMQConfigBean;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.threadpool.CyAsyncManager;
import org.rcisoft.core.util.*;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.business.cmsOrder.dao.CmsOrderRepository;
import org.rcisoft.business.cmsOrder.entity.CmsOrder;
import org.rcisoft.business.cmsOrder.service.CmsOrderService;

import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2025年1月6日 下午1:44:44.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CmsOrderServiceImpl extends ServiceImpl<CmsOrderRepository,CmsOrder>  implements CmsOrderService {

    private static final String ACTIVITY_ERR_MSG = "当前活动火爆， 请稍后重试";

    @Autowired(required = false)
    private CyRedisService cyRedisServiceImpl;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    RabbitClusterUtil rabbitClusterUtil;

    @Autowired
    RabbitMQConfigBean rabbitMQConfigBean;

    @Autowired
    private StockService stockService;

    @Autowired
    private CmsActivityRepository activityRepository;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    CyIdGenUtil cyIdGenUtil;

    @Autowired
    WxPayService wxPayService;

    /**
    * 保存 订单信息表
    * @param cmsOrder
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(CmsOrder cmsOrder){
        //增加操作
        int line = 1;
//        int line = baseMapper.insert(cmsOrder);
//        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
//            cmsOrder.getBusinessId()+"的订单信息表信息");
        //加redis锁
        boolean isGetLock = false;
        RLock lock = redissonClient.getLock(RedisCons.ACTIVITY_INFO + cmsOrder.getGoodsId());
        try {
            isGetLock = lock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                    Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
            if (isGetLock) {
                //获取redis【activity:detail:${aid}】
                Object detail = cyRedisServiceImpl.get(RedisCons.ACTIVITY_DETAIL +":"+ cmsOrder.getGoodsId());
                if (detail!=null){
                    CmsActivity info = JSON.parseObject(JSON.toJSONString(detail), CmsActivity.class);
                    if (info.getEndTime().before(new Date()))
                        throw new CyServiceException("活动已结束，不允许报名");
                    if (info.getStartTime().before(new Date()))
                        throw new CyServiceException("活动进行中，不允许报名");
                    //判断人数是否强制
                    CmsOrder orderInfo = new CmsOrder();
//                    if (info.getIsRestrictions().equals(0)){
                    mandatoryLimitNum(cmsOrder, info);
//                    } else { //人数非强制
//                        orderInfo = noMandatoryLimitNum(orderGoodsDto, info);
//                    }
                    //发送 mq订单信息
//                    rabbitClusterUtil.sendMsg(cmsOrder, rabbitMQConfigBean.ORDER_EXCHANGE_NAME,rabbitMQConfigBean.ORDER_ADD_QUEUE_NAME);
//                    if (orderInfo.getStatus().equals(0)){
//                        rabbitClusterUtil.sendMsg(cmsOrder, rabbitMQConfigBean.ORDER_EXCHANGE_DELAY_NAME,rabbitMQConfigBean.ORDER_CLOSE_QUEUE_NAME);
//                    }
//                    return cmsApplication;
                } else {
                    //key不存在 -> 返回   当前活动火爆 -> 请稍后重试
                    CyAsyncManager.me().execute(AsyncFactory.loadActivityRedis(cmsOrder.getGoodsId()));
                    throw new CyServiceException(ACTIVITY_ERR_MSG);
                }
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            isGetLock = false;
        } finally {
            lock.unlock();
            log.error("释放成功");
        }
        return new CyPersistModel(line);
    }

    //强制人数
    private void mandatoryLimitNum(CmsOrder orderInfo, CmsActivity info) {
        //执行lua脚本  decrby 【activity:count:${aid}】人数

        //判断人数是否大于=0
        Date endTime = TimeUtil.getTime(info.getEndTime(), Calendar.DATE,1);
        //将活动信息插入redis
        long time = TimeUtil.getTimeSecond(new Date(),endTime);
        long stock = stockService.stock(RedisCons.ACTIVITY_COUNT +":"+ orderInfo.getGoodsId(), time,  orderInfo.getNum(), () -> initStock(orderInfo.getGoodsId()));
        if (stock >= 0){ //报名成功
            long orderId = cyIdGenUtil.getSnowFlakedId();
//            OrderInfo orderInfo = new OrderInfo();
            orderInfo.setBusinessId(orderId);
            orderInfo.setName(info.getTitle());
//            orderInfo.setGoodsId(orderGoodsDto.getGoodsId());
            orderInfo.setUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
            orderInfo.setAccount(CyUserUtil.getAuthenUsername());
//            orderInfo.setNum(orderInfo.getNum());
            //计算价格 -开始
            if (info.getApplicationFee() != null && info.getApplicationFee().compareTo(BigDecimal.ZERO) > 0) {
                calculatePrice(orderInfo,info);
            } else { //免费 --> 支付完成
                orderInfo.setPrice(BigDecimal.ZERO);
                orderInfo.setPayPrice(BigDecimal.ZERO);
                orderInfo.setStatus(9);
            }
            orderInfo.setType(1);
            orderInfo.setOrderTime(new Date());
            orderInfo.setDelFlag("0");
            orderInfo.setCreateDate(new Date());
            orderInfo.setCreateBy(CyUserUtil.getAuthenBusinessId());
            orderInfo.setState(0);
//            orderInfo.setCouponId(orderGoodsDto.getCouponId());
//            orderInfo.setStoreId(info.getSource());
//            orderInfo.setActivityPrice(info.getApplicationFee());
            cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY +":"+ orderInfo.getGoodsId() + ":" + orderId,orderInfo);
//            return orderInfo;
        } else
            throw new CyServiceException("超出人数限制，报名失败");
    }

    //初始化redis  活动数
    private int initStock(long goodsId) {
        CmsActivity info = activityRepository.selectById(goodsId);
        //redis ttl 活动结束时间+1天
        Date endTime = TimeUtil.getTime(info.getEndTime(), Calendar.DATE, 1);
        //将活动信息插入redis
        long time = TimeUtil.getTimeSecond(new Date(), endTime);
//        if (info.getIsRestrictions().equals(0)) {
        AtomicReference<Integer> num = new AtomicReference<>(info.getMaxApplicationCount());
        //已报名人数计算： 数据库+ redis【order:activity:${所有}】 value 【hash】未取消 的并集
        QueryWrapper<CmsOrder> wrapper = new QueryWrapper<>();
        wrapper.eq("goods_id", goodsId);
        wrapper.notIn("status", 1, 2, 3, 4, 5, 6, 7, 10);
        wrapper.eq("del_flag", 0);
        List<CmsOrder> orderList = baseMapper.selectList(wrapper);
        for (CmsOrder orderInfo : orderList) {
            num.set(num.get() - orderInfo.getNum());
        }
        List<Long> orderIds = orderList.stream().map(CmsOrder::getBusinessId).toList();
        Set<String> map = redisTemplate.keys(RedisCons.ORDER_ACTIVITY + ":" + goodsId + "*");
        if (CollectionUtils.isNotEmpty(map)) {
            //遍历获取对应报名量
            map.forEach(m -> {
                Object detail = cyRedisServiceImpl.get(m);
                CmsOrder order = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                if (!orderIds.contains(order.getBusinessId()))
                    num.set(num.get() - order.getNum());
            });
        }
        cyRedisServiceImpl.set(RedisCons.ACTIVITY_COUNT + ":" + goodsId, num.get(), time);
        return num.get();
//        } else {
//            cyRedisServiceImpl.set(RedisCons.ACTIVITY_COUNT + ":" + goodsId, -1, time);
//            return -1;
//        }
    }

    //计算订单价格
    private void calculatePrice(CmsOrder orderInfo, CmsActivity info) {
        BigDecimal price = info.getApplicationFee().multiply(BigDecimal.valueOf(orderInfo.getNum())).setScale(2, BigDecimal.ROUND_DOWN);
        orderInfo.setStatus(0);

        //查询会员折扣
        BigDecimal discount = new BigDecimal("100.00");
//        discount = memberDiscount(discount);

//        if (orderGoodsDto.getCouponId() == null){
            orderInfo.setPrice(price);
            orderInfo.setPayPrice(price.multiply(discount.divide(BigDecimal.valueOf(100), 2, BigDecimal.ROUND_DOWN)).setScale(2, BigDecimal.ROUND_DOWN));
//        } else {
//            //先查询优惠券信息
//            Object couponObject = cyRedisServiceImpl.get(RedisCons.COUPON_DETAIL + ":" + orderGoodsDto.getCouponId());
//            if (couponObject != null) {
//                UserCouponRelative coupon = JSON.parseObject(JSON.toJSONString(couponObject), UserCouponRelative.class);
//                if (price.compareTo(coupon.getLimitAmount()) > -1) {
//                    if (coupon.getType().equals(0)) {
//                        //代金券 ---》   先使用代金券计算订单总价，再使用会员折扣计算支付总价
//                        price = price.subtract(coupon.getAmount()).setScale(2, BigDecimal.ROUND_DOWN);
//                        orderInfo.setPrice(price);
//                        //支付价格
//                        orderInfo.setPayPrice(price.multiply(discount.divide(BigDecimal.valueOf(100), 2, BigDecimal.ROUND_DOWN)).setScale(2, BigDecimal.ROUND_DOWN));
//                    } else {
//                        //折扣券  ---》   比较优惠券和会员折扣大小，使用最小折扣计算
//                        if (discount.compareTo(coupon.getDiscount()) > 0) {
//                            price = price.multiply(coupon.getDiscount().divide(BigDecimal.valueOf(100), 2, BigDecimal.ROUND_DOWN)).setScale(2, BigDecimal.ROUND_DOWN);
//                            orderInfo.setPrice(price);
//                            orderInfo.setPayPrice(price);
//                        } else {
//                            orderInfo.setPrice(price);
//                            orderInfo.setPayPrice(price.multiply(discount.divide(BigDecimal.valueOf(100), 2, BigDecimal.ROUND_DOWN)).setScale(2, BigDecimal.ROUND_DOWN));
//                        }
//                    }
//                } else {//如果商品总价大于等于代金券使用金额限制
//                    throw new CyServiceException("订单不符合满减条件，不能使用该优惠券");
//                }
//            } else {
//                //key不存在 -> 返回   当前活动火爆 -> 请稍后重试
//                CyAsyncManager.me().execute(AsyncFactory.loadCouponUserRedis(orderGoodsDto.getCouponId()));
//                throw new CyServiceException(ACTIVITY_ERR_MSG);
//            }
//        }
        if (orderInfo.getPayPrice().compareTo(BigDecimal.ZERO) <= 0){
            orderInfo.setStatus(9);
        }
    }

    /**
    * 删除 订单信息表
    * @param cmsOrder
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(CmsOrder cmsOrder){
        int line = baseMapper.realDelete(cmsOrder);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            cmsOrder.getBusinessId()+"的订单信息表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 订单信息表
    * @param cmsOrder
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(CmsOrder cmsOrder){
        cmsOrder.setDeleted();
        int line = baseMapper.deleteById(cmsOrder);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                cmsOrder.getBusinessId()+"的订单信息表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 订单信息表
    * @param cmsOrder
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(CmsOrder cmsOrder){
        int line = baseMapper.updateById(cmsOrder);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ cmsOrder.getBusinessId()+"的订单信息表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 订单信息表
    * @param id
    * @return
    */
    @Override
    public CmsOrder findById(String id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 订单信息表
    * @param cmsOrder
    * @return
    */
    @Override
    public IPage<CmsOrder> findAllByPagination(CyPageInfo<CmsOrder> paginationUtility,
        CmsOrder cmsOrder){
        return baseMapper.queryCmsOrdersPaged(paginationUtility,cmsOrder);

    }

    /**
    * 查询list 订单信息表
    * @param cmsOrder
    * @return
    */
    @Override
    public List<CmsOrder> findAll(CmsOrder cmsOrder){
        return baseMapper.queryCmsOrders(cmsOrder);

    }

    /**
    * 导出订单信息表
    * @return
    */
    @Override
    public List<CmsOrder> export(CmsOrder cmsOrder) {
        List<CmsOrder> cmsOrderList = baseMapper.queryCmsOrders(cmsOrder);
        return cmsOrderList;
    }

    /**
     * 活动取消
     * @param orderInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public int cancel(CmsOrder orderInfo) {
        //查询活动信息
        CmsActivity activity = activityRepository.selectById(orderInfo.getGoodsId());
        if (activity!=null) {
            if (activity.getEndTime().before(new Date()))
                throw new CyServiceException("活动已结束，不允许取消订单");
            if (activity.getStartTime().before(new Date()))
                throw new CyServiceException("活动进行中，不允许取消订单");
//            if (activity.getPrice() != null && activity.getPrice().compareTo(BigDecimal.ZERO) <= 0)
//                throw new CyServiceException("产生费用的订单才可以取消订单");
            //redisson key：【order:redis:${oid}】
            //加订单redis锁
            boolean isGetLock = false;
            RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderInfo.getBusinessId());
            try {
                isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                        Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                if (isGetLock) {
                    //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 已取消
                    Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY +":"+ orderInfo.getGoodsId() +":"+ orderInfo.getBusinessId());
                    if (detail!=null) {
                        CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                        Integer status = orderDetail.getStatus();
                        if (!Objects.equals(1,orderInfo.getManualRefund()) && status.equals(11))
                            throw new CyServiceException("已核销的订单不能取消订单");
                        if (status.equals(0) || status.equals(9) || status.equals(11)) { //如果订单是待支付、支付完成才能取消订单
                            orderDetail.setState(0);
                            orderDetail.setStatus(1);
                            cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY + ":" + orderInfo.getGoodsId() + ":" + orderInfo.getBusinessId(), orderDetail);
                            //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                            rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME, rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                            //lua脚本  incrby 【activity:count:${aid}】 人数
//                            if (activity.getIsRestrictions().equals(0))
                                stockService.addStock(RedisCons.ACTIVITY_COUNT +":"+ orderDetail.getGoodsId(), orderDetail.getNum());
                            //已支付的订单启动异步线程【线程池】 -> 调用退款接口
                            if ((status.equals(9) ||status.equals(11)) && orderDetail.getPayPrice().compareTo(BigDecimal.ZERO) > 0) {
                                orderDetail.setManualRefund(orderInfo.getManualRefund());
                                CyAsyncManager.me().execute(AsyncFactory.refundOrder(orderDetail));
                            }
                        }
                    }
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                isGetLock = false;
            } finally {
                redisLock.unlock();
                log.error("释放成功");
            }
        } else
            throw new CyServiceException("该活动不存在");
        return 1;
    }


    /**
     * 查询会员我的订单列表
     *
     * @param orderInfo 订单
     * @return 订单集合
     */
    @Override
    public IPage<CmsOrder> selectOrderUserByPagination(CyPageInfo<CmsOrder> paginationUtility,
                                                       CmsOrder orderInfo)
    {
        List<CmsOrder> orderRedis = new ArrayList<>();
        //查询edis + DB  ，redis 优先
        Set<String> map = redisTemplate.keys(RedisCons.ORDER_ACTIVITY+"*");
        if (CollectionUtils.isNotEmpty(map)) {
            for (String key : map) {
                Object detail = cyRedisServiceImpl.get(key);
                CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                //查询当前登录人的订单,如果是查询订单需要金额大于零，如果查询状态不为空需要过滤
                if (orderDetail.getUserId().equals(Integer.valueOf(CyUserUtil.getAuthenBusinessId())) &&
                        (orderInfo.getStatusList() == null || orderInfo.getStatusList().contains(orderDetail.getStatus()))){
                    orderRedis.add(orderDetail);
                }
            }
        }
        orderInfo.setUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        //查询数据库的数据
        List<CmsOrder> orderDbList = baseMapper.queryOrderUserList(orderInfo);
        //最终数据集合
        List<CmsOrder> list = orderRedis;
        List<Long> ids = list.stream().map(CmsOrder::getBusinessId).toList();
        for (CmsOrder info : orderDbList) {
            if (!ids.contains(info.getBusinessId()))
                list.add(info);
        }
        int pageNum = Math.toIntExact(paginationUtility.getCurrent());
        int pageSize = Math.toIntExact(paginationUtility.getSize());
        if (list.isEmpty()){
            IPage<CmsOrder> page = paginationUtility;
            page.setPages(pageNum);
            page.setSize(pageSize);
            page.setCurrent(pageNum);
            page.setTotal(list.size());
            page.setRecords(list);
            return page;
        }
        //下单时间倒叙
        list = list.stream().sorted(Comparator.comparing(CmsOrder::getOrderTime).
                reversed()).toList();
        //手动分页
        int total = list.size();
        if (total > pageSize) {
            int toIndex = pageSize * pageNum;
            if (toIndex > total) {
                toIndex = total;
            }
            list = list.subList(pageSize * (pageNum - 1), toIndex);
        }
        if (list.isEmpty()){
            IPage<CmsOrder> page = paginationUtility;
            page.setPages(pageNum);
            page.setSize(pageSize);
            page.setCurrent(pageNum);
            page.setTotal(list.size());
            page.setRecords(list);
            return page;
        }
        // 查询订单的其他信息
        List<CmsOrder> infos = baseMapper.queryOrderInfoByIds(list);
        for (CmsOrder info : list) {
            CmsOrder o = infos.stream().filter(l -> l.getBusinessId().equals(info.getBusinessId())).toList().get(0);
            info.setImgUrl(o.getImgUrl());
            info.setActivityBeginDate(o.getActivityBeginDate());
        }

        IPage<CmsOrder> page = paginationUtility;
        page.setPages(pageNum);
        page.setSize(pageSize);
        page.setCurrent(pageNum);
        page.setTotal(total);
        page.setRecords(list);
        return page;
    }



    /**
     * 退款确认
     * @param orderInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public int refundConfirm(CmsOrder orderInfo) throws IOException, JDOMException {
        //主动查询订单退款状态接口
        String result = wxPayService.selectWxOrderRefundStatus(orderInfo.getTransactionId());
        //退款状态：
        // SUCCESS—退款成功
        // REFUNDCLOSE—退款关闭，指商户发起退款失败的情况。
        // PROCESSING—退款处理中
        // CHANGE—退款异常，退款到银行发现用户的卡作废或者冻结了，导致原路退款银行卡失败，可前往商户平台（pay.weixin.qq.com）-交易中心，手动处理此笔退款。
        switch (result) {
            case "SUCCESS" -> { //退款成功
                log.info("SUCCESS--退款成功");
                //加订单redis锁
                boolean isGetLock = false;
                RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderInfo.getBusinessId());
                try {
                    isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                            Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                    if (isGetLock) {
                        Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY + ":" + orderInfo.getGoodsId() + ":" + orderInfo.getBusinessId());
                        if (detail != null) {
                            CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                            if (orderDetail.getStatus().equals(2) || orderDetail.getStatus().equals(4)) {
                                orderDetail.setState(0);
                                if (orderDetail.getStatus().equals(2))
                                    orderDetail.setStatus(6);
                                else if (orderDetail.getStatus().equals(4))
                                    orderDetail.setStatus(7);
                                //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 退款完成
                                cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY + ":" + orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(), orderDetail);
                                //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                                rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME, rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                            }
                        }
                    }
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    isGetLock = false;
                } finally {
                    redisLock.unlock();
                    log.error("释放成功");
                }
            }
            case "REFUNDCLOSE" -> {
                //退款关闭，指商户发起退款失败的情况。
                log.info("REFUNDCLOSE--退款关闭，指商户发起退款失败的情况。");
                throw new CyServiceException("商户发起退款失败，退款关闭");
            }
            case "PROCESSING" -> { //退款处理中
                log.info("PROCESSING—退款处理中");
                throw new CyServiceException("退款处理中");
            }
            case "CHANGE" -> { //退款异常
                log.info("CHANGE—退款异常");
                throw new CyServiceException("退款异常");
            }
            default -> {
                log.info("订单退款状态查询失败");
                throw new CyServiceException("订单退款关闭，商户发起退款失败");
            }
        }
        return 1;
    }



    /**
     * 支付确认
     * @param orderInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public int payConfirm(CmsOrder orderInfo) throws IOException, JDOMException {
        //主动查询订单支付状态接口
        String status = wxPayService.selectWxOrderStatus(String.valueOf(orderInfo.getBusinessId()));
        switch (status) {
            case "SUCCESS" -> { //支付成功
                log.info("SUCCESS--支付成功");
                //加订单redis锁
                boolean isGetLock = false;
                RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderInfo.getBusinessId());
                try {
                    isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                            Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                    if (isGetLock) {
                        Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY + ":" + orderInfo.getGoodsId() + ":" + orderInfo.getBusinessId());
                        if (detail != null) {
                            CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                            if (orderDetail.getStatus().equals(8)) {
                                orderDetail.setState(0);
                                orderDetail.setStatus(9);
                                //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 支付完成
                                cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY + ":" + orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(), orderDetail);
                                //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                                rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME, rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                            }
                        }
                    }
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    isGetLock = false;
                } finally {
                    redisLock.unlock();
                    log.error("释放成功");
                }
            }
            case "REFUND" -> { //转入退款
                log.info("REFUND--转入退款");
                throw new CyServiceException("订单转入退款");
            }
            case "NOTPAY" -> { //未支付
                log.info("NOTPAY--未支付");
                //加订单redis锁
                boolean isGetLock = false;
                RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderInfo.getBusinessId());
                try {
                    isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                            Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                    if (isGetLock) {
                        Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY + ":" + orderInfo.getGoodsId() + ":" + orderInfo.getBusinessId());
                        if (detail != null) {
                            CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                            if (orderDetail.getStatus().equals(8)) {
                                orderDetail.setState(0);
                                orderDetail.setStatus(0);
                                //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 支付完成
                                cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY + ":" + orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(), orderDetail);
                                //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                                rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME, rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                            }
                        }
                    }
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    isGetLock = false;
                } finally {
                    redisLock.unlock();
                    log.error("释放成功");
                }
            }
            case "CLOSED" -> { //已关闭
                log.info("CLOSED--已关闭");
                throw new CyServiceException("订单已关闭");
            }
            case "REVOKED" -> { //已撤销(刷卡支付)
                log.info("REVOKED--已撤销(刷卡支付)");
                throw new CyServiceException("订单已撤销");
            }
            case "USERPAYING" -> { //用户支付中
                log.info("USERPAYING--用户支付中");
                throw new CyServiceException("用户支付中");
            }
            case "PAYERROR" -> { //支付失败(其他原因，如银行返回失败)
                log.info("PAYERROR--支付失败(其他原因，如银行返回失败)");
                //加订单redis锁
                boolean isGetLock = false;
                RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderInfo.getBusinessId());
                try {
                    isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                            Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                    if (isGetLock) {
                        Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY +":"+ orderInfo.getGoodsId() +":"+ orderInfo.getBusinessId());
                        if (detail!=null) {
                            CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                            //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 待支付
                            orderDetail.setState(0);
                            orderDetail.setStatus(0);
                            cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY +":"+ orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId(),orderDetail);
                            //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                            rabbitClusterUtil.sendMsg(orderDetail, rabbitMQConfigBean.ORDER_EXCHANGE_NAME,rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                        } else
                            throw new CyServiceException("订单支付失败，请稍后再试");
                    }
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    isGetLock = false;
                } finally {
                    redisLock.unlock();
                    log.error("释放成功");
                }
            }
            case "ACCEPT" -> { //已接收，等待扣款
                log.info("ACCEPT--已接收，等待扣款");
                throw new CyServiceException("已接收，等待扣款");
            }
            default -> {
                log.info("订单状态查询失败");
                throw new CyServiceException("订单状态查询失败");
            }
        }
        return 1;
    }


    /**
     * 查询异常订单列表
     *
     * @param orderInfo 订单
     * @return 订单集合
     */
    @Override
    public IPage<CmsOrder> selectErrorOrderByPagination(CyPageInfo<CmsOrder> paginationUtility,
                                                        CmsOrder orderInfo)
    {
        //查询DB，缺省查询，近4个月的活动，如今日是2023-5-5，查询 2023-1-5至 2023-4-5 的活动
//        Date now = new Date();
//        Date startTime =  TimeUtil.getTime(now, Calendar.MONTH,-4)
//        Date endTime =  TimeUtil.getTime(now, Calendar.MONTH,-1)
//        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd")
        QueryWrapper<CmsActivity> wrapper = new QueryWrapper<>();
        wrapper.eq("del_flag",0);
        wrapper.between("end_time",orderInfo.getBeginTime(),orderInfo.getEndTime());
        List<CmsActivity> activityInfos = activityRepository.selectList(wrapper);
        List<CmsOrder> list = new ArrayList<>();
        for (CmsActivity activity : activityInfos) {
            //[order:activity:${aid}:*] 匹配，返回
            Set<String> map = redisTemplate.keys(RedisCons.ORDER_ACTIVITY+":"+activity.getBusinessId()+"*");
            if (CollectionUtils.isNotEmpty(map)) {
                for (String key : map) {
                    Object detail = cyRedisServiceImpl.get(key);
                    CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                    list.add(orderDetail);
                }
            }
        }
        int pageNum = Math.toIntExact(paginationUtility.getCurrent());
        int pageSize = Math.toIntExact(paginationUtility.getSize());
        if (list.isEmpty()){
            IPage<CmsOrder> page = paginationUtility;
            page.setPages(pageNum);
            page.setSize(pageSize);
            page.setCurrent(pageNum);
            page.setTotal(0);
            page.setRecords(list);
            return page;
        }
        //下单时间倒叙
        list = list.stream().sorted(Comparator.comparing(CmsOrder::getOrderTime).
                reversed()).toList();
        //手动分页
        int total = list.size();
        if (total > pageSize) {
            int toIndex = pageSize * pageNum;
            if (toIndex > total) {
                toIndex = total;
            }
            list = list.subList(pageSize * (pageNum - 1), toIndex);
        }
        if (list.isEmpty()){
            IPage<CmsOrder> page = paginationUtility;
            page.setPages(pageNum);
            page.setSize(pageSize);
            page.setCurrent(pageNum);
            page.setTotal(list.size());
            page.setRecords(list);
            return page;
        }
        IPage<CmsOrder> page = paginationUtility;
        page.setPages(pageNum);
        page.setSize(pageSize);
        page.setCurrent(pageNum);
        page.setTotal(total);
        page.setRecords(list);
        return page;
    }


    /**
     * 活动退款补偿
     * @param ids
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public int refundReimbursement(List<Long> ids) {
        //退款补偿【退款接口调用失败3次以上的】
        for (Long id : ids) {
            CmsOrder orderDb = baseMapper.selectById(id);
            if ((!orderDb.getStatus().equals(3) && !orderDb.getStatus().equals(5)) || orderDb.getRefundCount() < 3)
                throw new CyServiceException("退款调用失败3次以上的才可以进行退款补偿");
            //调用退款接口
            String result = wxPayService.refundOrder(orderDb);
            //redisson key：【order:redis:${oid}】
            boolean isGetLock = false;
            RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderDb.getBusinessId());
            try {
                isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                        Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                if (isGetLock) {
                    Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY +":"+ orderDb.getGoodsId() +":"+ orderDb.getBusinessId());
                    if (detail!=null) {
                        CmsOrder orderRedis = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                        orderRedis.setState(0);
                        if ("SUCCESS".equals(result)) { //接口成功
                            orderRedis.setStatus(2);
                        } else { //接口失败
                            orderRedis.setStatus(3);
                            Integer refundCount = orderRedis.getRefundCount();
                            orderRedis.setRefundCount(refundCount + 1);
                        }
                        //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 退款中 (2 3)
                        cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY + ":" + orderRedis.getGoodsId() + ":" + orderRedis.getBusinessId(), orderRedis);
                        //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                        rabbitClusterUtil.sendMsg(orderRedis, rabbitMQConfigBean.ORDER_EXCHANGE_NAME, rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME);
                    }
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                isGetLock = false;
            } finally {
                redisLock.unlock();
                log.error("释放成功");
            }
        }
        return 1;
    }


    /**
     * 异常订单手动补偿
     * @param dto
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public int manualCompensateOrder(CmsOrder dto) {
        List<OrderActivityDto> list = dto.getDtos();
        for (OrderActivityDto orderActivityDto : list) {
            //查询DB -> 是否已落库
            CmsOrder info = baseMapper.selectById(orderActivityDto.getOrderId());
            //未落库->落库: redis key: [order:activity:$faid+:$foid] [末同步] =>落库
            Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY + ":" + orderActivityDto.getActivityId() + ":" + orderActivityDto.getOrderId());
            if (detail != null) {
                CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                if (orderDetail.getState().equals(0)) {
                    if (info == null) {
                        baseMapper.insert(orderDetail);
                        //如果有优惠券，进行核销
//                        if (orderDetail.getCouponId() != null) {
//                            UserCouponRelative userCouponRelative = new UserCouponRelative();
//                            userCouponRelative.setBusinessId(orderDetail.getCouponId());
//                            userCouponRelative.setOrderId(orderDetail.getBusinessId());
//                            userCouponRelative.setStatus(1);
//                            userCouponRelative.setCancelId(orderDetail.getStoreId());
//                            userCouponRelative.setCancelTime(new Date());
//                            couponInfoRepository.useCouponById(userCouponRelative);
//                        }
                        boolean isGetLock = false;
                        //加订单redis锁 redisson key: [order:redis:${oid}]
                        RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderDetail.getBusinessId());
                        try {
                            isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                                    Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                            if (isGetLock) {
                                //del redis key: 【order:activity:${aid}:${oid}】
                                cyRedisServiceImpl.del(RedisCons.ORDER_ACTIVITY + ":" + orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId());
                            }
                        } catch (InterruptedException e) {
                            Thread.currentThread().interrupt();
                            isGetLock = false;
                        } finally {
                            redisLock.unlock();
                            log.error("释放成功");
                        }
                    } else {
                        //0 待支付   1已取消   2退款中   3退款中   4后台退款中   5后台退款中   6退款完成   7后台退款完成   8支付中   9 支付完成   10订单关闭
                        switch (orderDetail.getStatus()) {
                            case 0, 2, 3, 4, 5, 6, 7, 8, 11 -> orderUpdateStatus(orderDetail);
                            case 1 ->//1已取消
                                    orderCancel(orderDetail);
                            case 9 ->//支付完成
                                    orderFinish(orderDetail);
                            default -> {
                            }
                        }
                    }
                }
            }
        }
        return 1;
    }

    //订单状态改为支付中
    private void orderUpdateStatus(CmsOrder orderDetail) {
        boolean isGetLock = false;
        orderDetail.setUpdateDate(new Date());
        baseMapper.updateById(orderDetail);
        //加订单redis锁
        RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderDetail.getBusinessId());
        try {
            isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                    Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
            if (isGetLock) {
                //del redis key: 【order:activity:${aid}:${oid}】
                cyRedisServiceImpl.del(RedisCons.ORDER_ACTIVITY +":"+ orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId());
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            isGetLock = false;
        } finally {
            redisLock.unlock();
            log.error("释放成功");
        }
    }

    //订单状态改为已取消
    private void orderCancel(CmsOrder orderDetail) {
        boolean isGetLock = false;
        orderDetail.setUpdateDate(new Date());
        baseMapper.updateById(orderDetail);
        //订单优惠券退还
//        if (orderDetail.getCouponId() != null) {
//            UserCouponRelative userCouponRelative = new UserCouponRelative();
//            userCouponRelative.setBusinessId(orderDetail.getCouponId());
//            userCouponRelative.setStatus(0);
//            couponInfoRepository.cancelCouponById(userCouponRelative);
//        }
        //加订单redis锁
        RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderDetail.getBusinessId());
        try {
            isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                    Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
            if (isGetLock) {
                //del redis key: 【order:activity:${aid}:${oid}】
                cyRedisServiceImpl.del(RedisCons.ORDER_ACTIVITY +":"+ orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId());
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            isGetLock = false;
        } finally {
            redisLock.unlock();
            log.error("释放成功");
        }
    }

    private void orderFinish(CmsOrder orderDetail) {
        boolean isGetLock = false;
        orderDetail.setUpdateDate(new Date());
        baseMapper.updateById(orderDetail);
        //积分计算
//        BigDecimal price = orderDetail.getPayPrice();
//        IntegralAmountRelative relative = integralAmountRelativeRepository.queryIntegralByPrice(price);
//        if (relative != null) {
//            //查询会员信息
//            SysUser info = sysUserRepositorys.selectUserById(orderDetail.getUserId());
//            UserIntegral userIntegral = new UserIntegral();
//            userIntegral.setUserId(orderDetail.getUserId());
//            userIntegral.setGoodsId(orderDetail.getGoodsId());
//            userIntegral.setChange(relative.getIntegral());
//            Integer integral = info.getIntegral() + relative.getIntegral();
//            userIntegral.setIntegral(integral);
//            userIntegral.setCreateBy(String.valueOf(orderDetail.getUserId()));
//            userIntegral.setCreateTime(new Date());
//            userIntegralRepository.insertUserIntegral(userIntegral);
//            //会员等级计算
//            MemberBenefits benefits = memberBenefitsRepository.selectMemberBenefitsByLevel(info.getLevel()+1);
//            if (benefits != null && (integral >= benefits.getIntegral())) {
//                info.setLevel(info.getLevel()+1);
//            }
//            info.setIntegral(integral);
//            sysUserRepositorys.updateSysUser(info);
//            cyRedisServiceImpl.set(RedisCons.MEMBER_INFO + ":" + orderDetail.getUserId(), info);
//        }
        //加订单redis锁
        RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderDetail.getBusinessId());
        try {
            isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                    Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
            if (isGetLock) {
                //del redis key: 【order:activity:${aid}:${oid}】
                cyRedisServiceImpl.del(RedisCons.ORDER_ACTIVITY +":"+ orderDetail.getGoodsId() + ":" + orderDetail.getBusinessId());
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            isGetLock = false;
        } finally {
            redisLock.unlock();
            log.error("释放成功");
        }
    }

}
