package org.rcisoft.business.memInfo.controller;


/*固定导入*/
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.Operation;
import org.rcisoft.business.memInfo.entity.ArticleExamStatus;
import org.rcisoft.business.memInfo.entity.opmArticleDTO;
import org.rcisoft.business.memInfo.entity.peopleNumberDTO;
import org.rcisoft.core.anno.CyEncryptSm4Anno;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import jakarta.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.business.memInfo.service.MemInfoService;

import java.util.List;

/**
* Created by cy on 2024年12月20日 下午3:35:34.
*/
@RestController
@RequestMapping("/memInfo")
public class MemInfoController extends CyPaginationController<MemInfo> {


    @Autowired
    private MemInfoService memInfoServiceImpl;

    @PreAuthorize("@cyPerm.hasPerm('mem:info:add')")
    @CyOpeLogAnno(title = "system-会员表管理-新增会员表", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary="添加会员表", description="添加会员表")
    @PostMapping(value = "/add")
    @CyEncryptSm4Anno
    public CyResult add(@Valid MemInfo memInfo, BindingResult bindingResult) {
        CyPersistModel data = memInfoServiceImpl.persist(memInfo);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        memInfo);
    }

    @PreAuthorize("@cyPerm.hasPerm('mem:info:delete')")
    @CyOpeLogAnno(title = "system-会员表管理-删除会员表", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary="逻辑删除会员表", description="逻辑删除会员表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true,  schema = @Schema(type = "string"))})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId,MemInfo memInfo) {
        memInfo.setBusinessId(businessId);
        CyPersistModel data = memInfoServiceImpl.removeLogical(memInfo);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }
    @PreAuthorize("@cyPerm.hasPerm('mem:info:delete')")
    @CyOpeLogAnno(title = "system-会员表管理-删除会员表", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary="删除会员表", description="删除会员表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true,  schema = @Schema(type = "string"))})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,MemInfo memInfo) {
        memInfo.setBusinessId(businessId);
        CyPersistModel data = memInfoServiceImpl.remove(memInfo);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    @PreAuthorize("@cyPerm.hasPerm('mem:info:update')")
    @CyOpeLogAnno(title = "system-会员表管理-修改会员表", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary="修改会员表", description="修改会员表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false,  schema = @Schema(type = "string"))})
    @PutMapping("/update/{businessId:\\w+}")
//    @CyEncryptSm4Anno
    public CyResult update(@PathVariable int businessId, @Valid MemInfo memInfo, BindingResult bindingResult) {
        memInfo.setBusinessId(businessId);
        CyPersistModel data = memInfoServiceImpl.merge(memInfo);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        memInfo);
    }

    @PreAuthorize("@cyPerm.hasPerm('mem:info:query')")
    @CyOpeLogAnno(title = "system-会员表管理-查询会员表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询单一会员表", description="查询单一会员表")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true,  schema = @Schema(type = "string"))})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        memInfoServiceImpl.findById(businessId));
    }

    @PreAuthorize("@cyPerm.hasPerm('mem:info:list')")
    @CyOpeLogAnno(title = "system-会员表管理-查询会员表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询会员表集合", description="查询会员表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(MemInfo memInfo) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            memInfoServiceImpl.findAll(memInfo));
    }

    @PreAuthorize("@cyPerm.hasPerm('mem:info:list')")
    @CyOpeLogAnno(title = "system-会员表管理-查询会员表", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="分页查询会员表集合", description="分页查询会员表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(MemInfo memInfo) {
        memInfoServiceImpl.findAllByPagination(getPaginationUtility(), memInfo);
        return getGridModelResponse();
    }

    @PreAuthorize("@cyPerm.hasPerm('mem:info:export')")
    @CyOpeLogAnno(title = "system-会员表管理-查询会员表", businessType = CyLogTypeEnum.EXPORT)
    @Operation(summary = "导出会员表信息", description = "导出会员表信息")
    @GetMapping(value = "/export")
    public void outMemInfo(HttpServletResponse response,MemInfo memInfo,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="会员表信息.xls";break;
            case "1": excelName="会员表信息.xlsx";break;
            case "2": excelName="会员表信息.csv";break;
        }
        List<MemInfo> memInfoList = memInfoServiceImpl.export(memInfo);
        CyEpExcelUtil.exportExcel(memInfoList, "会员表信息", "会员表信息", MemInfo.class, excelName, response);
    }

    @CyOpeLogAnno(title = "system-会员表管理-校验手机号", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary="校验手机号", description="校验手机号")
    @PostMapping(value = "/checkInfo")
    @CyEncryptSm4Anno
    public CyResult checkInfo(@Valid MemInfo memInfo) {
        return CyResultGenUtil.genSuccessResult(memInfoServiceImpl.checkInfo(memInfo));
    }

    @PreAuthorize("@cyPerm.hasPerm('mem:info:query')")
    @CyOpeLogAnno(title = "system-用户管理-查询用户身份证、手机号、真实姓名", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "查询用户身份证、手机号、真实姓名", description = "查询用户身份证、手机号、真实姓名")
    @GetMapping(value = {"/getUserPhone"})
    public CyResult getUserPhone(Integer businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                memInfoServiceImpl.getUserPhone(businessId));
    }

    @PreAuthorize("@cyPerm.hasPerm('mem:info:list')")
    @Operation(summary="分页查询动态表集合", description="分页查询动态表集合")
    @GetMapping(value = "/listArticle")
    public CyGridModel listArticleByPagination(opmArticleDTO opmArticleDTO) {
        memInfoServiceImpl.findArticleByPagination(getPaginationUtility(), opmArticleDTO);
        return getGridModelResponse();
    }

    /*
    *   查询会员人数、实名人数、报名人数、文章数
    */
    @GetMapping(value = "/getPeopleNumber")
    public CyResult getPeopleNumber(peopleNumberDTO peopleNumberDTO) {
        return CyResultGenUtil.genSuccessResult(memInfoServiceImpl.getPeopleNumber(peopleNumberDTO));
    }

    /**
     * 查询文章审核
     */
    @GetMapping(value = "/getArticle")
    public CyResult getArticle(ArticleExamStatus articleExamStatus) {
        return CyResultGenUtil.genSuccessResult(memInfoServiceImpl.getArticle(articleExamStatus));
    }


}
