package org.rcisoft.business.memInfo.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.cmsActivity.entity.CmsActivity;
import org.rcisoft.business.memInfo.bean.MemberGenerateBean;
import org.rcisoft.business.memInfo.entity.ArticleExamStatus;
import org.rcisoft.business.memInfo.entity.opmArticleDTO;
import org.rcisoft.business.memInfo.entity.peopleNumberDTO;
import org.rcisoft.core.constant.CyDelStaCons;
import org.rcisoft.core.constant.CyFlagStaCons;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.util.CyAESUtils;
import org.rcisoft.core.util.CyIdGenUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.business.memInfo.service.MemInfoService;
import org.rcisoft.core.service.CyBaseService;

import org.rcisoft.sys.constant.CyUserCons;
import org.rcisoft.sys.rbac.user.dao.SysUserRbacRepository;
import org.rcisoft.sys.rbac.user.dto.SysUserRbacDTO;
import org.rcisoft.sys.rbac.user.entity.SysUserRbac;
import org.rcisoft.sys.rbac.user.enums.UserInfoExceptionEnums;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.time.LocalDate;
import java.time.Month;
import java.util.*;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2024年12月20日 下午3:35:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class MemInfoServiceImpl extends ServiceImpl<MemInfoRepository,MemInfo>  implements MemInfoService {

    // 定义允许的字符集
    private static final String CHAR_POOL = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_";
    private static final Random RANDOM = new Random();
    private static final Set<String> generatedNicknames = new HashSet<>();

    @Autowired
    private SysUserRbacRepository sysUserRbacRepository;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    CyIdGenUtil cyIdGenUtil;
    @Autowired
    MemberGenerateBean memberGenerateBean;

    @Value("${cy.init.password}")
    private String password;


    /**
    * 保存 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(MemInfo memInfo){
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.encrypt(memInfo.getMemRealName()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(null,memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(null,memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        //添加sys_user
        SysUserRbac userRbac = new SysUserRbac();
        userRbac.setUsername(this.generateRandomName());
        userRbac.setUserType(CyUserCons.USER_TYPE_COMMON);
        userRbac.setPhone(memInfo.getMemPhone());
        userRbac.setSex(memInfo.getMemSex());
        userRbac.setPassword(passwordEncoder.encode(password));
        userRbac.setIdNumber(memInfo.getMemIdcard());
        userRbac.setUserIdentity("10");
        userRbac.setFlag(CyFlagStaCons.NORMAL.getStatus());
        userRbac.setDelFlag(CyDelStaCons.NORMAL.getStatus());
        userRbac.setWxOpenid(memInfo.getWxOpenid());
//        userRbac.setAvatar(memInfo.getAvatar());
        userRbac.setNickName(memInfo.getMemNickName());
        sysUserRbacRepository.insert(userRbac);
        //查询会员角色
        List<Integer> roleIds = baseMapper.selectRoleByKey("member");
        //插入角色
        if (!roleIds.isEmpty()) {
            sysUserRbacRepository.insertUserRole(userRbac.getBusinessId(), roleIds);
        }
        //生成会员号
        String maxCode = baseMapper.selectMaxCode();
        String memCode = generateMemCode(maxCode);
        memInfo.setMemCode(memCode);
        //增加操作
        memInfo.setUserId(userRbac.getBusinessId());
        memInfo.setSort(0);
        memInfo.setIsRecommended(0);
        int line = baseMapper.insert(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(MemInfo memInfo){
        int line = baseMapper.realDelete(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(MemInfo memInfo){
        memInfo.setDeleted();
        int line = baseMapper.deleteById(memInfo);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                memInfo.getBusinessId()+"的会员表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(MemInfo memInfo){
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.encrypt(memInfo.getMemRealName()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(memInfo.getUserId(),memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(memInfo.getUserId(),memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        //修改sys_user
        SysUserRbac userRbac = new SysUserRbac();
        userRbac.setBusinessId(memInfo.getUserId());
        userRbac.setPhone(memInfo.getMemPhone());
        userRbac.setSex(memInfo.getMemSex());
        userRbac.setIdNumber(memInfo.getMemIdcard());
//        userRbac.setAvatar(memInfo.getAvatar());
        userRbac.setNickName(memInfo.getMemNickName());
        sysUserRbacRepository.updateById(userRbac);
        int line = baseMapper.updateById(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 会员表
    * @param id
    * @return
    */
    @Override
    public MemInfo findById(int id){
        return baseMapper.selectDetailById(id);
    }

    /**
    * 分页查询 会员表
    * @param memInfo
    * @return
    */
    @Override
    public IPage<MemInfo> findAllByPagination(CyPageInfo<MemInfo> paginationUtility,
        MemInfo memInfo){
        if (memInfo.getMinAge()!=null && memInfo.getMaxAge()!=null) {
            memInfo.setBeginBirthday(computeBirthdayBegin(memInfo.getMaxAge()));
            memInfo.setEndBirthday(computeBirthdayEnd(memInfo.getMinAge()));
        }
        return baseMapper.queryMemInfosPaged(paginationUtility,memInfo);

    }

    /**
    * 查询list 会员表
    * @param memInfo
    * @return
    */
    @Override
    public List<MemInfo> findAll(MemInfo memInfo){
        return baseMapper.queryMemInfos(memInfo);

    }

    /**
    * 导出会员表
    * @return
    */
    @Override
    public List<MemInfo> export(MemInfo memInfo) {
        List<MemInfo> memInfoList = baseMapper.queryMemInfos(memInfo);
        return memInfoList;
    }


    private String generateRandomName() {
        StringBuilder nickname;
        int length = 10;
        do {
            nickname = new StringBuilder(length);
            for (int i = 0; i < length; i++) {
                int index = RANDOM.nextInt(CHAR_POOL.length());
                nickname.append(CHAR_POOL.charAt(index));
            }
        } while (generatedNicknames.contains(nickname.toString()));
        // 存储生成的昵称
        generatedNicknames.add(nickname.toString());
        return "wx_" + nickname.toString();
    }


    /**
     * 校验会员信息
     * @param memInfo
     * @return
     */
    @Override
    public Boolean checkInfo(MemInfo memInfo) {
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(memInfo.getUserId(),memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(null,memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        return true;
    }

    /**
     * 通过年龄计算出生日期开始
     * @param maxAge
     * @return
     */
    private static String computeBirthdayBegin(Integer maxAge) {
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 计算出生年份：当前年份减去年龄-1
        int birthYear = currentDate.getYear() - maxAge - 1;
        // 计算出生日期：当前日期晚一天
        LocalDate birthDate = currentDate.plusDays(1);
        birthDate = birthDate.withYear(birthYear);
        return birthDate.toString();
    }


    /**
     * 通过年龄计算出生日期结束
     * @param minAge
     * @return
     */
    private static String computeBirthdayEnd(Integer minAge) {
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 计算出生年份：当前年份减去年龄-1
        int birthYear = currentDate.getYear() - minAge;
        // 计算出生日期：当前日期晚一天
        LocalDate birthDate = currentDate.withYear(birthYear);
        return birthDate.toString();
    }

    /**
     * 生成会员号
     * @param code
     * @return
     */
    private String generateMemCode(String code) {  //将会员号段排序
        List<String> codeList = Arrays.stream(memberGenerateBean.getCodes()).sorted().collect(Collectors.toList());
        if (StringUtils.isNotBlank(code)) {
            int currentNumber = Integer.parseInt(code);
            //获取当前会员号所处的号段
            for (int i = 0; i < codeList.size(); i++) {
                String[] range = codeList.get(i).split("-");
                int minCode = Integer.parseInt(range[0]);
                int maxCode = Integer.parseInt(range[1]);
                if (currentNumber >= minCode && currentNumber <= maxCode) {
                    // 如果当前号码在号段内，返回该号段中的下一个号码
                    int nextNumber = currentNumber + 1;
                    if (nextNumber <= maxCode) {
                        // 获取当前号码的长度
                        int length = code.length();
                        // 格式化为4位数（根据需要调整位数）
                        return String.format("%0" + length + "d", nextNumber);
                    } else {
                        // 当前号码已经是该号段的最大值，返回下一个号段的起始值
                        if ((i + 1) < codeList.size())
                            return codeList.get(i + 1).split("-")[0];
                        throw new CyServiceException("会员号超出设定最大值，请联系管理员");
                    }
                }
            }
            return null;
        } else {
            return codeList.get(0).split("-")[0];
        }
    }



    @Override
    public MemInfo getUserPhone(Integer businessId) {
        MemInfo memInfo = baseMapper.getUserPhone(businessId);
        //存储加密信息解密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.decrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.decrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.decrypt(memInfo.getMemRealName()));
        return memInfo;
    }


    @Override
    public IPage<opmArticleDTO> findArticleByPagination(CyPageInfo<MemInfo> paginationUtility,
                                                        opmArticleDTO opmArticleDTO){
        return baseMapper.queryOpmArticlesPaged(paginationUtility,opmArticleDTO);

    }

    @Override
    public peopleNumberDTO getPeopleNumber(peopleNumberDTO peopleNumberDTO ){
        peopleNumberDTO list = new peopleNumberDTO();

        list.setArticleNumber(baseMapper.getArticleNumber(peopleNumberDTO));
        list.setActivityNumber(baseMapper.getActivityNumber(peopleNumberDTO));
        list.setRealNumber(baseMapper.getRealNumber(peopleNumberDTO));
        list.setRegisterNumber(baseMapper.getRegisterNumber(peopleNumberDTO));
        list.setCmsActivityList(baseMapper.getCmsActivityList(peopleNumberDTO));
        list.setVisitCount(baseMapper.getCmsActivityList(peopleNumberDTO));

        return list;
    }


    @Override
    public ArticleExamStatus getArticle(ArticleExamStatus articleExamStatus ){

        LocalDate startDate = articleExamStatus.getDate().minusDays(7);
        LocalDate startDateMonth = articleExamStatus.getDate().minusMonths(7);
        LocalDate endDate = articleExamStatus.getDate();

        ArticleExamStatus list = new ArticleExamStatus();

        //审核通过
        list.setArticleExamStatusDayYes(baseMapper.getArticleCountByDateRangeYes(startDate, endDate));
        list.setArticleExamStatusMonthYes(baseMapper.getArticleCountByMonthRangeYes(startDateMonth, endDate));

        //审核未通过
        list.setArticleExamStatusDayNo(baseMapper.getArticleCountByDateRangeNo(startDate, endDate));
        list.setArticleExamStatusMonthNo(baseMapper.getArticleCountByMonthRangeNo(startDateMonth, endDate));

        return list;
    }
}
