package org.rcisoft.business.opmArticle.controller;


/*固定导入*/

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import jakarta.servlet.http.HttpServletResponse;
import org.rcisoft.business.opmArticle.entity.LikeDTO;
import org.rcisoft.business.opmArticle.entity.OpmArticle;
import org.rcisoft.business.opmArticle.service.OpmArticleService;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
* Created by cy on 2024年3月30日 下午1:51:15.
*/
@RestController
@RequestMapping("/opmarticle")
public class OpmArticleController extends CyPaginationController<OpmArticle> {


    @Autowired
    private OpmArticleService opmArticleServiceImpl;

    @PreAuthorize("@cyPerm.hasPerm('cms:opmArticle:increase')")
    @CyOpeLogAnno(title = "system-opmArticle管理管理-新增opmArticle管理", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary="添加opmArticle管理", description="添加opmArticle管理")
    @PostMapping(value = "/add")
    public CyResult add(@Valid @RequestBody OpmArticle opmArticle, BindingResult bindingResult) {
        CyPersistModel data = opmArticleServiceImpl.persist(opmArticle);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
                opmArticle);
    }
    @PreAuthorize("@cyPerm.hasPerm('cms:opmArticle:removing')")
    @CyOpeLogAnno(title = "system-opmArticle管理管理-删除opmArticle管理", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary="逻辑删除opmArticle管理", description="逻辑删除opmArticle管理")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable String businessId,OpmArticle opmArticle) {
        opmArticle.setBusinessId(Integer.valueOf(businessId));
        CyPersistModel data = opmArticleServiceImpl.removeLogical(opmArticle);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:opmArticle:delete')")
    @CyOpeLogAnno(title = "system-opmArticle管理管理-删除opmArticle管理", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary="删除opmArticle管理", description="删除opmArticle管理")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable String businessId,OpmArticle opmArticle) {
        opmArticle.setBusinessId(Integer.valueOf(businessId));
        CyPersistModel data = opmArticleServiceImpl.remove(opmArticle);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:opmArticle:modifications')")
    @CyOpeLogAnno(title = "system-opmArticle管理管理-修改opmArticle管理", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary="修改opmArticle管理", description="修改opmArticle管理")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false)})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable String businessId, @Valid OpmArticle opmArticle, BindingResult bindingResult) {
        opmArticle.setBusinessId(Integer.valueOf(businessId));
        CyPersistModel data = opmArticleServiceImpl.merge(opmArticle);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
                opmArticle);
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:opmArticle:singleSearch')")
    @CyOpeLogAnno(title = "system-opmArticle管理管理-查询opmArticle管理", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询单一opmArticle管理", description="查询单一opmArticle管理")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable String businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
                opmArticleServiceImpl.findById(businessId));
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:opmArticle:search')")
    @CyOpeLogAnno(title = "system-opmArticle管理管理-查询opmArticle管理", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="查询opmArticle管理集合", description="查询opmArticle管理集合")
    @GetMapping(value = "/queryOpmArticle")
    public CyResult queryOpmArticle(OpmArticle opmArticle) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
                opmArticleServiceImpl.findAll(opmArticle));
    }

//    @PreAuthorize("@cyPerm.hasPerm('cms:opmArticle:pageSearch')")
    @CyOpeLogAnno(title = "system-opmArticle管理管理-查询opmArticle管理", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary="分页查询opmArticle管理集合", description="分页查询opmArticle管理集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(OpmArticle opmArticle) {
        opmArticleServiceImpl.findAllByPagination(getPaginationUtility(), opmArticle);
        return getGridModelResponse();
    }

    @PreAuthorize("@cyPerm.hasPerm('cms:opmArticle:derive')")
    @CyOpeLogAnno(title = "system-opmArticle管理管理-查询opmArticle管理", businessType = CyLogTypeEnum.EXPORT)
    @Operation(summary = "导出opmArticle管理信息", description = "导出opmArticle管理信息")
    @GetMapping(value = "/export")
    public void outOpmArticle(HttpServletResponse response, OpmArticle opmArticle, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="opmArticle管理信息.xls";break;
            case "1": excelName="opmArticle管理信息.xlsx";break;
            case "2": excelName="opmArticle管理信息.csv";break;
        }
        List<OpmArticle> opmArticleList = opmArticleServiceImpl.export(opmArticle);
        CyEpExcelUtil.exportExcel(opmArticleList, "opmArticle管理信息", "opmArticle管理信息", OpmArticle.class, excelName, response);
    }

    /**
     * 逻辑删除评论
     */
    @PreAuthorize("@cyPerm.hasPerm('cms:opmArticle:removing')")
    @DeleteMapping("/deleteComment/{businessId:\\w+}")
    public CyResult deleteComment(@PathVariable String businessId,OpmArticle opmArticle) {
        opmArticle.setBusinessId(Integer.valueOf(businessId));
        CyPersistModel data = opmArticleServiceImpl.removeComment(opmArticle);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }


    /**
     * 点赞增加点赞数
     */
    @PostMapping("/likeAdd/{articleId:\\w+}")
    public CyResult likeAdd(@PathVariable String articleId) {
        CyPersistModel data = opmArticleServiceImpl.likeAdd(articleId);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                articleId);
    }
}
