package org.rcisoft.business.opmArticle.service.impl;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.opmArticle.dao.OpmArticleRepository;
import org.rcisoft.business.opmArticle.entity.ArticleCommentDTO;
import org.rcisoft.business.opmArticle.entity.LikeDTO;
import org.rcisoft.business.opmArticle.entity.OpmArticle;
import org.rcisoft.business.opmArticle.service.OpmArticleService;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import java.util.List;

/**
 * Created by cy on 2024年3月30日 下午1:51:15.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class OpmArticleServiceImpl extends ServiceImpl<OpmArticleRepository, OpmArticle> implements OpmArticleService {

    /**
    * 保存 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(OpmArticle opmArticle){
        //增加操作
        int line = baseMapper.insert(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
                opmArticle.getBusinessId()+"的opmArticle管理信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(OpmArticle opmArticle){
        int line = baseMapper.realDelete(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
                opmArticle.getBusinessId()+"的opmArticle管理信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(OpmArticle opmArticle){
        opmArticle.setDeleted();
        int line = baseMapper.deleteById(opmArticle);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                    opmArticle.getBusinessId()+"的opmArticle管理信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(OpmArticle opmArticle){
        int line = baseMapper.updateById(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ opmArticle.getBusinessId()+"的opmArticle管理信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 opmArticle管理
    * @param id
    * @return
    */
    @Override
    public OpmArticle findById(String id){
        OpmArticle opmArticle = baseMapper.selectByIdWithUrl(id);
        List<ArticleCommentDTO> list = baseMapper.SelectArticleComment(opmArticle.getBusinessId());
        List<String> pathList = baseMapper.SelectPathList(opmArticle.getBusinessId());
        opmArticle.setPath(pathList);
        opmArticle.setArticleCommentDTOList(list);
        return opmArticle;
    }

    /**
    * 分页查询 opmArticle管理
    * @param opmArticle
    * @return
    */
    @Override
    public IPage<OpmArticle> findAllByPagination(CyPageInfo<OpmArticle> paginationUtility,
                                                 OpmArticle opmArticle){
        IPage<OpmArticle> result = baseMapper.queryOpmArticlePaged(paginationUtility,opmArticle);
        return result;

    }

    /**
    * 查询list opmArticle管理
    * @param opmArticle
    * @return
    */
    @Override
    public List<OpmArticle> findAll(OpmArticle opmArticle){
        return baseMapper.queryOpmArticle(opmArticle);

    }

    /**
    * 导出opmArticle管理
    * @return
    */
    @Override
    public List<OpmArticle> export(OpmArticle opmArticle) {
        List<OpmArticle> opmArticleList = baseMapper.queryOpmArticle(opmArticle);
        return opmArticleList;
    }


    /**
     * 逻辑删除 评论
     * @param opmArticle
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeComment(OpmArticle opmArticle){
        opmArticle.setDeleted();
        int line = baseMapper.deleteComment(opmArticle);
        log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                opmArticle.getBusinessId()+"的opmArticleComment信息");
        return new CyPersistModel(line);
    }

    /**
     * 点赞增加点赞数
     * @param articleId
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel likeAdd(String articleId){
        //增加操作
        LikeDTO likeDTO = new LikeDTO();
        likeDTO.setUserId(CyUserUtil.getAuthenBusinessId());
        likeDTO.setArticleId("1");
        int line = baseMapper.likeAdd(likeDTO);
        return new CyPersistModel(line);
    }

}
