package org.rcisoft.business.opmTopic.controller;


/*固定导入*/

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import jakarta.servlet.http.HttpServletResponse;
import org.rcisoft.business.opmTopic.entity.OpmTopic;
import org.rcisoft.business.opmTopic.service.OpmTopicService;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * @author cy
 * @date 2024年3月25日 下午1:42:40
 */
@RestController
@RequestMapping("/opmTopic")
public class OpmTopicController extends CyPaginationController<OpmTopic> {

    @Autowired
    private OpmTopicService opmTopicServiceImpl;

    @PreAuthorize("@cyPerm.hasPerm('opm:topic:increase')")
    @CyOpeLogAnno(title = "system-话题管理-新增话题", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary = "添加话题", description = "添加话题")
    @PostMapping(value = "/add")
    public CyResult add(@Valid OpmTopic opmTopic, BindingResult bindingResult) {
        CyPersistModel data = opmTopicServiceImpl.persist(opmTopic);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                opmTopic);
    }

    @PreAuthorize("@cyPerm.hasPerm('opm:topic:removing')")
    @CyOpeLogAnno(title = "system-话题管理-删除话题", businessType = CyLogTypeEnum.DELETE)
    @Operation(summary = "删除话题", description = "删除话题")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable Integer businessId, OpmTopic opmTopic) {
        opmTopic.setBusinessId(businessId);
        CyPersistModel data = opmTopicServiceImpl.remove(opmTopic);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @PreAuthorize("@cyPerm.hasPerm('opm:topic:modifications')")
    @CyOpeLogAnno(title = "system-话题管理-修改话题", businessType = CyLogTypeEnum.INSERT)
    @Operation(summary = "修改话题",  description = "修改话题")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = false)})
    @PostMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable Integer businessId, @Valid OpmTopic opmTopic, BindingResult bindingResult) {
        opmTopic.setBusinessId(businessId);
        CyPersistModel data = opmTopicServiceImpl.merge(opmTopic);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                opmTopic);
    }

//    @PreAuthorize("@cyPerm.hasPerm('opm:notice:singleSearch')")
    @CyOpeLogAnno(title = "system-话题管理-查询话题", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "查询单一话题", description = "查询单一话题")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable Integer businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                opmTopicServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:contentNewsInformation:list')")
    @CyOpeLogAnno(title = "system-话题管理-查询话题", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "查询话题集合", description = "查询话题集合")
    @GetMapping(value = "/queryAllOpmTopic")
    public CyResult queryAllTopicList(OpmTopic opmTopic) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                opmTopicServiceImpl.findAll(opmTopic));
    }

//    @PreAuthorize("@cyPerm.hasPerm('opm:topic:pageSearch')")
    @CyOpeLogAnno(title = "system-话题管理-查询话题", businessType = CyLogTypeEnum.QUERY)
    @Operation(summary = "分页查询话题集合", description = "分页查询话题集合")
    @GetMapping(value = "/queryOpmTopicByPagination")
    public CyGridModel listByPagination(OpmTopic opmTopic) {
        opmTopicServiceImpl.findAllByPagination(getPaginationUtility(), opmTopic);
        return getGridModelResponse();
    }

    //修改状态
    @PreAuthorize("@cyPerm.hasPerm('opm:topic:modificationsStatus')")
    @CyOpeLogAnno(title = "system-话题管理-修改话题状态", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary = "修改话题状态", description = "修改话题状态")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @PostMapping("/updateStatus/{businessId:\\w+}")
    public CyResult updateStatus(OpmTopic opmTopic) {
        CyPersistModel data = opmTopicServiceImpl.updateStatus(opmTopic);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                opmTopic);
    }


    //修改审核状态
    @CyOpeLogAnno(title = "system-话题管理-修改话题状态", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary = "修改话题状态", description = "修改话题状态")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @PostMapping("/updateExamStatus/{businessId:\\w+}")
    public CyResult updateExamStatus(OpmTopic opmTopic) {
        CyPersistModel data = opmTopicServiceImpl.updateExamStatus(opmTopic);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                opmTopic);
    }
    //修改审核状态
    @CyOpeLogAnno(title = "system-话题管理-修改话题状态", businessType = CyLogTypeEnum.UPDATE)
    @Operation(summary = "修改话题状态", description = "修改话题状态")
    @Parameters({@Parameter(name = "businessId", description = "businessId", required = true)})
    @PostMapping("/updateNoExamStatus/{businessId:\\w+}")
    public CyResult updateNoExamStatus(OpmTopic opmTopic) {
        CyPersistModel data = opmTopicServiceImpl.updateNoExamStatus(opmTopic);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                opmTopic);
    }
    @PreAuthorize("@cyPerm.hasPerm('opm:topic:derive')")
    @CyOpeLogAnno(title = "system-话题管理-查询话题", businessType = CyLogTypeEnum.EXPORT)
    @Operation(summary = "导出话题信息", description = "导出话题信息")
    @GetMapping(value = "/export")
    public void outTopicInformation(HttpServletResponse response, OpmTopic opmTopic, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "话题信息.xls";
                break;
            case "1":
                excelName = "话题信息.xlsx";
                break;
            case "2":
                excelName = "话题信息.csv";
                break;
        }
        List<OpmTopic> opmTopicList = opmTopicServiceImpl.export(opmTopic);
        CyEpExcelUtil.exportExcel(opmTopicList, "话题信息", "话题信息", OpmTopic.class, excelName, response);
    }
}
