package org.rcisoft.core.async;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.NoArgsConstructor;
import org.rcisoft.app.pay.service.WxPayService;
import org.rcisoft.business.cmsActivity.dao.CmsActivityRepository;
import org.rcisoft.business.cmsActivity.entity.CmsActivity;
import org.rcisoft.business.cmsOrder.dao.CmsOrderRepository;
import org.rcisoft.business.cmsOrder.entity.CmsOrder;
import org.rcisoft.core.component.CySpringBeanComp;
import org.rcisoft.core.rabbitmq.bean.RabbitMQConfigBean;
import org.rcisoft.core.service.impl.CyRedisServiceImpl;
import org.rcisoft.core.util.RabbitClusterUtil;
import org.rcisoft.core.util.RedisCons;
import org.rcisoft.core.util.TimeUtil;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 异步工厂（产生任务用）
 *
 * @author isoft
 */
@NoArgsConstructor
public class AsyncFactory
{

    private static final Logger log = LoggerFactory.getLogger(AsyncFactory.class);


    //加载redis优惠券数量
//    public static TimerTask loadCouponCountRedis(final Long id) {
//        return new TimerTask() {
//            @Override
//            public void run() {
//                // 查询优惠券剩余数量，加载redis
//                CouponInfo info = CySpringBeanComp.getBean(CouponInfoRepository.class).selectCouponInfoById(id);
//                Integer count = CySpringBeanComp.getBean(CouponInfoRepository.class).selectCountById(id);
//                Integer num = info.getNum() - count;
//                CySpringBeanComp.getBean(CyRedisServiceImpl.class).set(RedisCons.COUPON_COUNT +":"+ id,num);
//            }
//        };
//    }

    //加载活动信息、数量
    public static TimerTask loadActivityRedis(final Integer id) {
        return new TimerTask() {
            @Override
            public void run() {
                CmsActivity info = CySpringBeanComp.getBean(CmsActivityRepository.class).selectById(id);
                //redis ttl 活动结束时间+1天
                Date endTime = TimeUtil.getTime(info.getEndTime(), Calendar.DATE, 1);
                //将活动信息插入redis
                long time = TimeUtil.getTimeSecond(new Date(), endTime);
                CySpringBeanComp.getBean(CyRedisServiceImpl.class).set(RedisCons.ACTIVITY_DETAIL + ":" + id, info, time);
//                if (info.getIsRestrictions().equals(0)) {
                    AtomicReference<Integer> num = new AtomicReference<>(info.getMaxApplicationCount());
                    //已报名人数计算： 数据库+ redis【order:activity:${所有}】 value 【hash】未取消 的并集
                    QueryWrapper<CmsOrder> wrapper = new QueryWrapper<>();
                    wrapper.eq("activity_id", id);
                    wrapper.notIn("status", 1, 2, 3, 4, 5, 6, 7, 10);
                    wrapper.eq("del_flag", 0);
                    List<CmsOrder> orderList = CySpringBeanComp.getBean(CmsOrderRepository.class).selectList(wrapper);
                    for (CmsOrder orderInfo : orderList) {
                        num.set(num.get() - orderInfo.getNum());
                    }
                    List<Long> orderIds = orderList.stream().map(CmsOrder::getBusinessId).toList();
                    Set<String> map = CySpringBeanComp.getBean(RedisTemplate.class).keys(RedisCons.ORDER_ACTIVITY + ":" + id + "*");
                    if (CollectionUtils.isNotEmpty(map)) {
                        //遍历获取对应报名量
                        map.forEach(m -> {
                            Object detail = CySpringBeanComp.getBean(CyRedisServiceImpl.class).get(m);
                            CmsOrder order = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                            if (!orderIds.contains(order.getBusinessId()))
                                num.set(num.get() - order.getNum());
                        });
                    }
                    CySpringBeanComp.getBean(CyRedisServiceImpl.class).set(RedisCons.ACTIVITY_COUNT + ":" + id, num.get(), time);
//                } else {
//                    CySpringBeanComp.getBean(CyRedisServiceImpl.class).set(RedisCons.ACTIVITY_COUNT + ":" + id, -1, time);
//                }
            }
        };
    }


//    //加载redis会员信息
//    public static TimerTask loadMemberRedis(final Long id) {
//        return new TimerTask() {
//            @Override
//            public void run() {
//                // 查询会员信息，加载redis
//                SysUser info = CySpringBeanComp.getBean(SysUserRepositorys.class).selectUserById(id);
//                CySpringBeanComp.getBean(CyRedisServiceImpl.class).set(RedisCons.MEMBER_INFO +":"+ id,info);
//            }
//        };
//    }
//
//    //加载redis会员信息
//    public static TimerTask loadBenefitsRedis(final Integer level) {
//        return new TimerTask() {
//            @Override
//            public void run() {
//                // 查询会员权益信息，加载redis
//                MemberBenefits info = CySpringBeanComp.getBean(MemberBenefitsRepository.class).selectMemberBenefitsByLevel(level);
//                CySpringBeanComp.getBean(CyRedisServiceImpl.class).set(RedisCons.BENEFITS_INFO +":"+ level,info);
//            }
//        };
//    }

    //加载redis用户优惠券信息
//    public static TimerTask loadCouponUserRedis(final Long id) {
//        return new TimerTask() {
//            @Override
//            public void run() {
//                // 查询用户优惠券信息，加载redis
//                UserCouponRelative info = CySpringBeanComp.getBean(CouponInfoRepository.class).selectUserCouponRelativeById(id);
//                info.setBusinessList(CySpringBeanComp.getBean(CouponInfoRepository.class).selectBusinessIds(id));
//                Date endTime = TimeUtil.getTime(info.getUseEndTime(), Calendar.DATE,1);
//                long time = TimeUtil.getTimeSecond(new Date(),endTime);
//                if (time > 0)
//                    CySpringBeanComp.getBean(CyRedisServiceImpl.class).set(RedisCons.COUPON_DETAIL +":"+ id,info,time);
//            }
//        };
//    }

    //异步调用wx退款接口
    public static TimerTask refundOrder(final CmsOrder orderInfo) {
        return new TimerTask() {
            @Override
            public void run() {
                //调用退款接口
                String result = CySpringBeanComp.getBean(WxPayService.class).refundOrder(orderInfo);
                //redisson key：【order:redis:${oid}】
                boolean isGetLock = false;
                RLock redisLock = CySpringBeanComp.getBean(RedissonClient.class).getLock(RedisCons.ORDER_REDIS + orderInfo.getBusinessId());
                try {
                    isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                            Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                    if (isGetLock) {
                        orderInfo.setRefundTime(new Date());
                        orderInfo.setState(0);
                        if ("SUCCESS".equals(result)){ //接口成功
                            if (Objects.equals(orderInfo.getManualRefund(), 1))
                                orderInfo.setStatus(4);
                            else
                                orderInfo.setStatus(2);
                        } else { //接口失败
                            if (Objects.equals(orderInfo.getManualRefund(), 1))
                                orderInfo.setStatus(5);
                            else
                                orderInfo.setStatus(3);
                            orderInfo.setRefundCount(1);
                        }
                        //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【未同步】，status -> 退款中 (2 3)
                        CySpringBeanComp.getBean(CyRedisServiceImpl.class).set(RedisCons.ORDER_ACTIVITY +":"+ orderInfo.getGoodsId() + ":" + orderInfo.getBusinessId(),orderInfo);
                        //发送 mq，exchange：`order_activity`    route-key：`order_activity_status`   value 【订单】
//                        CySpringBeanComp.getBean(RabbitClusterUtil.class).sendMsg(orderInfo,
//                                CySpringBeanComp.getBean(RabbitMQConfigBean.class).ORDER_EXCHANGE_NAME,
//                                CySpringBeanComp.getBean(RabbitMQConfigBean.class).ORDER_STATUS_QUEUE_NAME);
                    }
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    isGetLock = false;
                } finally {
                    redisLock.unlock();
                    log.error("释放成功");
                }
            }
        };
    }
}
