package org.rcisoft.core.rabbitmq;

/**
 * rabbitmq
 *
 */

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.cmsActivity.dao.CmsActivityRepository;
import org.rcisoft.business.cmsActivity.entity.CmsActivity;
import org.rcisoft.business.cmsActivity.service.impl.StockService;
import org.rcisoft.business.cmsOrder.dao.CmsOrderRepository;
import org.rcisoft.business.cmsOrder.entity.CmsOrder;
import org.rcisoft.core.rabbitmq.bean.RabbitMQConfigBean;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.RedisCons;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class OrderActivityCloseRabbitListener {



    @Autowired
    RabbitMQConfigBean rabbitMQConfigBean;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired(required = false)
    private CyRedisService cyRedisServiceImpl;

    @Autowired
    private StockService stockService;

    @Autowired
    private CmsOrderRepository orderRepository;

//    @Autowired
//    private CouponInfoRepository couponInfoRepository;

    @Autowired
    private CmsActivityRepository activityInfoRepository;

    /**
     * 1.2 mq流程参考
     * <p>
     */

    @Transactional(propagation = Propagation.REQUIRED)
    @RabbitListener(queues = "${spring.rabbitmq.orderActivityCloseQueue:orderActivityCloseQueue}")
    public void directConsumer(String msg1, Channel channel, Message message) throws IOException {
        try {
            log.error("---orderActivityCloseQueue:"+msg1);
            String msg = new String(((Message) message).getBody(), StandardCharsets.UTF_8);
            CmsOrder orderInfo = JSONUtil.toBean(msg, CmsOrder.class);
            log.error("---CmsOrder:"+orderInfo);

            //加redis锁
            boolean isGetLock = false;
            RLock lock = redissonClient.getLock(RedisCons.ORDER_DB + orderInfo.getBusinessId());
            try {
                isGetLock = lock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                        Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                if (isGetLock) {
                    // redis  key: 【order:activity:${oid}】 value 【hash】；hash state   是否是 【待支付
                    Object detail = cyRedisServiceImpl.get(RedisCons.ORDER_ACTIVITY +":"+ orderInfo.getGoodsId() +":"+ orderInfo.getBusinessId());
                    if (detail!=null) {
                        CmsOrder orderDetail = JSON.parseObject(JSON.toJSONString(detail), CmsOrder.class);
                        if (orderDetail.getState().equals(0) && orderDetail.getStatus().equals(0)) {
                            //如果订单待支付，则关闭订单
                            orderDetail.setStatus(10);
                            orderDetail.setUpdateDate(new Date());
                            //查询DB -> 是否已落库
                            CmsOrder info = orderRepository.selectById(orderInfo.getBusinessId());
                            if (info != null)
                                orderRepository.updateById(orderDetail);
                            else
                                orderRepository.insert(orderDetail);
                            //如果使用了优惠券，则改为未使用
//                            if (orderDetail.getCouponId() != null) {
//                                UserCouponRelative userCouponRelative = new UserCouponRelative();
//                                userCouponRelative.setBusinessId(orderDetail.getCouponId());
//                                userCouponRelative.setStatus(0);
//                                couponInfoRepository.cancelCouponById(userCouponRelative);
//                            }
                            //加订单redis锁
                            RLock redisLock = redissonClient.getLock(RedisCons.ORDER_REDIS + orderInfo.getBusinessId());
                            try {
                                isGetLock = redisLock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                                        Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                                if (isGetLock) {
                                    //redis  key: 【order:activity:${aid}:${oid}】 value 【hash】；ttl：-1；hash state-> 【已同步】
                                    orderDetail.setState(1);
                                    cyRedisServiceImpl.set(RedisCons.ORDER_ACTIVITY +":"+ orderInfo.getGoodsId() + ":" + orderInfo.getBusinessId(),orderDetail);
                                    //退人数
//                                    CmsActivity activityInfo = activityInfoRepository.selectById(orderInfo.getActivityId());
                                    stockService.addStock(RedisCons.ACTIVITY_COUNT +":"+ orderDetail.getGoodsId(), orderDetail.getNum());
                                }
                            } catch (InterruptedException e) {
                                Thread.currentThread().interrupt();
                                isGetLock = false;
                            } finally {
                                redisLock.unlock();
                                log.error("释放成功");
                            }
                        }

                    }
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                isGetLock = false;
            } finally {
                lock.unlock();
                log.error("释放成功");
            }
        } catch (RuntimeException e) {
            log.error(e.getMessage());
        }
        finally {
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        }
    }




}
