package org.rcisoft.core.rabbitmq.config;

import org.rcisoft.core.rabbitmq.bean.RabbitMQConfigBean;
import org.springframework.amqp.core.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;


/**
 * RabbitMq配置类
 *
 */
@Configuration
public class RabbitMQClusterConfig {

    public static final long MIN =  60 * 1000L;

    private static final String X_MESSAGE_TTL = "x-message-ttl";
    private static final String X_DEAD_LETTER_EXCHANGE = "x-dead-letter-exchange";
    private static final String X_DEAD_LETTER_ROUTING_KEY = "x-dead-letter-routing-key";

    @Autowired
    RabbitMQConfigBean rabbitMQConfigBean;


    /**
     * 以下 消息队列
     * @return
     */
    // 创建持久化的Exchange
    @Bean
    Exchange orderActivityExchange() {
        return ExchangeBuilder.directExchange(rabbitMQConfigBean.ORDER_EXCHANGE_NAME).durable(true).build();
    }

    // 创建持久化的死信Exchange
    @Bean
    Exchange orderActivityDelayExchange() {
        return ExchangeBuilder.directExchange(rabbitMQConfigBean.ORDER_EXCHANGE_DELAY_NAME).durable(true).build();
    }


    // 创建持久化的queue
    @Bean
    Queue orderActivityAddQueue() {
        return  QueueBuilder.durable(rabbitMQConfigBean.ORDER_ADD_QUEUE_NAME).build();
    }

    // 创建持久化的queue
    @Bean
    Queue orderActivityStatusQueue() {
        return  QueueBuilder.durable(rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME).build();
    }


    // 创建持久化的queue
    @Bean
    Queue orderActivityCloseQueue() {
        return QueueBuilder.durable(rabbitMQConfigBean.ORDER_CLOSE_QUEUE_NAME).build();
    }

    // 创建持久化的queue
    @Bean
    Queue orderActivityCloseDelayQueue() {
        return QueueBuilder.durable(rabbitMQConfigBean.ORDER_CLOSE_QUEUE_DELAY_NAME)
                .withArgument(X_MESSAGE_TTL, 30 * MIN)
                .withArgument(X_DEAD_LETTER_EXCHANGE,rabbitMQConfigBean.ORDER_EXCHANGE_NAME)
                .withArgument(X_DEAD_LETTER_ROUTING_KEY,rabbitMQConfigBean.ORDER_CLOSE_QUEUE_NAME).build();
//                .ttl(25000)
//                .deadLetterExchange(rabbitMQConfigBean.ORDER_EXCHANGE_NAME)
//                .deadLetterRoutingKey(rabbitMQConfigBean.ORDER_CLOSE_QUEUE_NAME).build();
    }

    // 创建绑定
    @Bean
    Binding orderActivityAddBindings(){
        return BindingBuilder.bind(orderActivityAddQueue()).to(orderActivityExchange()).with(rabbitMQConfigBean.ORDER_ADD_QUEUE_NAME).and(null);
    }

    // 创建绑定
    @Bean
    Binding orderActivityStatusBindings(){
        return BindingBuilder.bind(orderActivityStatusQueue()).to(orderActivityExchange()).with(rabbitMQConfigBean.ORDER_STATUS_QUEUE_NAME).and(null);
    }

    // 创建绑定
    @Bean
    Binding orderActivityCloseBindings(){
        return BindingBuilder.bind(orderActivityCloseQueue()).to(orderActivityExchange()).with(rabbitMQConfigBean.ORDER_CLOSE_QUEUE_NAME).and(null);
    }

    // 创建绑定
    @Bean
    Binding orderActivityCloseDelayBindings(){
        return BindingBuilder.bind(orderActivityCloseDelayQueue()).to(orderActivityDelayExchange()).with(rabbitMQConfigBean.ORDER_CLOSE_QUEUE_NAME).and(null);
    }
}
