package org.rcisoft.core.rabbitmq.config;


import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.config.Config;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;


//@ConditionalOnProperty(prefix = "cy.model", name = {"redis"}, havingValue = "true")
@Configuration
@Slf4j
public class RedissonConfig {

    @Value("${spring.data.redis.host}")
    private String redisIP;

    @Value("${spring.data.redis.port}")
    private String redisPort;

    @Value("${spring.data.redis.password}")
    private String redisPassword;

    private Config config = new Config();


    @Bean(destroyMethod = "shutdown")
    public RedissonClient init() {
        try {
            //设置编解码器（默认FST）
            //config.setCodec(new JsonJacksonCodec());
            /**
             *
             * ERROR o.r.c.h.CommandDecoder.decodeCommand.134 - Unable to decode data. reply: +PONG
             * , channel: [id: 0x2bcaea2d, L:/192.168.1.74:53284 - R:117.122.212.91/117.122.212.91:26070], command: CommandData [promise=RedissonPromise [promise=ImmediateEventExecutor$ImmediatePromise@7490414a(failure: io.netty.util.concurrent.DefaultPromise$StacklessCancellationException)], command=(PING), params=[], codec=org.redisson.client.codec.StringCodec]
             * java.util.concurrent.CancellationException: null
             *
             * 发现并不是解码器的问题，debug一停，就会有该问题
             */
            //config.setCodec(new StringCodec());

            config.useSingleServer().setAddress("redis://" + redisIP + ":" + redisPort).setConnectTimeout(30000)//同任何节点建立连接时的等待超时。时间单位是毫秒。
                    .setTimeout(3000)//等待节点回复命令的时间。该时间从命令发送成功时开始计时。
                    .setRetryInterval(3000)//当与某个节点的连接断开时，等待与其重新建立连接的时间间隔。时间单位是毫秒。
                    .setPingConnectionInterval(1000)
                    .setPassword(StringUtils.isAnyEmpty(redisPassword) ? null : redisPassword);
            return Redisson.create(config);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return null;
    }


}
