package org.rcisoft.core.util;


import cn.hutool.json.JSONUtil;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.core.RabbitTemplate.ConfirmCallback;
import org.springframework.amqp.rabbit.core.RabbitTemplate.ReturnsCallback;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import java.util.UUID;

//mq
@Configuration
public class RabbitClusterUtil {
    private static final Logger log = LoggerFactory.getLogger(RabbitClusterUtil.class);

    @Autowired(required = false)
    RabbitTemplate rabbitTemplate;

//    public RabbitTemplate rabbitTemplate() {
////        RabbitTemplate template = new RabbitTemplate(connectionFactory());
//        return template;
//    }


    final ConfirmCallback confirmCallback = new ConfirmCallback() {
        //初始化
        public void confirm(CorrelationData correlationData, boolean ack, String cause) {
            if (!ack) {
                RabbitClusterUtil.log.error("异常处理....correlationData: " + correlationData);
            }

        }
    };
    final ReturnsCallback returnCallback = new ReturnsCallback() {
        //初始化
        public void returnedMessage(ReturnedMessage returnedMessage) {
            RabbitClusterUtil.log.error("return exchange: " + returnedMessage.getExchange() + ", routingKey: " +
                    returnedMessage.getRoutingKey() + ", replyCode: " + returnedMessage.getReplyCode() + ", replyText: " + returnedMessage.getReplyText());
        }
    };

    public RabbitClusterUtil() {
    }

    public void sendMsg(Object msg, String exchangeName, String routeKey) {
        //向mq发送消息
        String msgId = UUID.randomUUID().toString();
        this.rabbitTemplate.setConfirmCallback(this.confirmCallback);
        this.rabbitTemplate.setReturnsCallback(this.returnCallback);
        MessageProperties props = (MessageProperties)MessagePropertiesBuilder.newInstance().setDeliveryMode(MessageDeliveryMode.fromInt(2)).build();
        Message message = new Message(msg == null ? "".getBytes() : JSONUtil.toJsonStr(msg).getBytes(), props);
        this.rabbitTemplate.convertAndSend(exchangeName, routeKey, message, new CorrelationData(msgId));
    }
}
